from PIL import Image

def cut_image_in(input_path, output_path=None, target_size_WH=(256, 256)):
    # 打开图片
    image = Image.open(input_path)

    # 获取图片的宽度和高度
    width, height = image.size
    if width < height:
        # 高度是长边，宽度按比例缩放
        new_height = target_size_WH[1]
        new_width = int((target_size_WH[1] / height)*width)
    else:
        # 宽度是长边，高度按比例缩放
        new_width = target_size_WH[0]
        new_height = int((target_size_WH[0] / width)*height)

    cropped_image = image.resize((new_width, new_height), Image.LANCZOS)

    # print([width,height],[new_width, new_height])

    # 创建一个空白的256x256的画布
    final_image = Image.new("RGB", (target_size_WH[0], target_size_WH[1]), (255, 255, 255))
    # 将裁剪后的图片粘贴到画布中间
    final_image.paste(cropped_image, ((target_size_WH[0] - new_width) // 2, (target_size_WH[1] - new_height) // 2))

    # 保存裁剪后的图片
    if output_path:
        final_image.save(output_path)

    # print(final_image.size)

    return final_image

def cut_image_out(input_path, output_path=None, target_size_WH=(256, 256)):
    # 打开图片
    image = Image.open(input_path)

    # 获取图片的宽度和高度
    width, height = image.size
    if width < height:
        # 高度是长边，宽度按比例缩放
        new_width = target_size_WH[0]
        new_height = int((target_size_WH[0] / width)*height)
    else:
        # 宽度是长边，高度按比例缩放
        new_height = target_size_WH[1]
        new_width = int((target_size_WH[1] / height)*width)

    cropped_image = image.resize((new_width, new_height), Image.LANCZOS)

    print([width,height],[new_width, new_height])

    # 创建一个空白的256x256的画布
    final_image = Image.new("RGB", (target_size_WH[0], target_size_WH[1]), (255, 255, 255))
    # 将裁剪后的图片粘贴到画布中间
    final_image.paste(cropped_image, ((target_size_WH[0] - new_width) // 2, (target_size_WH[1] - new_height) // 2))

    # 保存裁剪后的图片
    if output_path:
      final_image.save(output_path)
    
    # print(final_image.size)

    return final_image


SETTING = {}

def getSetting(key=None, defaultValue=None):
  """获取配置"""
  return SETTING.get(key, defaultValue)

def applySetting(key:str|dict, value=None):
  """设置配置"""
  if isinstance(key, dict):
    keys = list(key.keys())
    for each in keys:
      SETTING[each] = key[each]
  else:
    SETTING[key] = value
    
  return SETTING


import os
def assetsPath(*args):
    """获取资源路径
    Args:
        args (tuple): 目录，文件名

    Returns:
        path (str): 资源绝对路径
    """
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), "assets", *args)

tips_ui = assetsPath("tips.ui")
tipsBox_ui = assetsPath("tipsBox.ui")
MainWindow_ui = assetsPath("main.ui")
loading_icon = assetsPath("loading.gif")
small_page_icon = assetsPath("small_page.png")
maximize_page_icon = assetsPath("maximize_page.png")

# 定义软件当前版本
APP_VERSION = '1.0.0'
APP_TITLE = '小灰妙记'
APP_ICON_PATH = assetsPath('favicon.png')

IS_DEBUG = False        #生产环境，使用远程服务器

PING_HOST = 'greatnote.cn'

filter_id = "leftBar"
button_frame_id = "frame_13"
button_container_id = "leftbar_container"
button_close_id = "btn_close"
button_login_id = "btn_login_icon"
button_name_id = "btn_login_text"

# 默认主题
default_theme = {
  "skin": {
    "current_skin_index": 0
  },
  "skinStyle": [
    {
      "id": "skin001",
      "name": "\u9ed8\u8ba4",
      "header_bg_color": "#6a5acd",
      "main_bg_color": "#f0f0f0",
      "app_bg_image": assetsPath("skin", "app_bg_image_1.png"),
    },
    {
      "id": "skin002",
      "name": "\u73ca\u745a\u7ea2",
      "header_bg_color": "#cd5a5a",
      "main_bg_color": "#fffbcb",
      "app_bg_image": assetsPath("skin", "app_bg_image_2.png"),
    },
    {
      "id": "skin003",
      "name": "\u70ab\u9177\u9ed1",
      "header_bg_color": "#000",
      "main_bg_color": "#e4d5ff",
      "app_bg_image": assetsPath("skin", "app_bg_image_3.png"),
    }
  ]
}


import time, os
from PySide6.QtWidgets import QFileDialog
from .cut_image import cut_image_out

def select_image(target, dist, title="选择背景图片", size=(1920, 1080)):
  imgName, imgType = QFileDialog.getOpenFileName(target, title, "", "*.jpg;;*.png;;All Files(*)")
  if not os.path.exists(dist):
    os.mkdir(dist)
  output = f'{dist}/{int(time.time()*1000)}.{imgName.split(".")[-1]}'
  abs_output = os.path.abspath(output)
  cut_image_out(imgName, abs_output, size)
  return output, abs_output


from PySide6.QtWidgets import QVBoxLayout, QWidget
from PySide6.QtCore import Qt
from PySide6.QtGui import QCloseEvent
from app_page_core import Callback, Param
from ..animation import FadeEffect, MoveWin, Shadow
from ..core import Setting
from ..utils import loadUI
from ..config import tipsBox_ui


class TipsBox(QWidget):
  def __init__(self, system_param:Param, msg_dict:dict={"topic":"更新提醒","title":"提示窗的标题","content":"提示的内容"}):
    super().__init__()
    self.callback = Callback()
    MoveWin(self, system_param, id="tips_box_position")

    ui = Setting.getSetting("tipsBox_ui", tipsBox_ui)
    # 如果ui是字符串
    if isinstance(ui, str):
        self.ui = loadUI(ui)
        # 创建垂直布局
        layout = QVBoxLayout(self)
        layout.addWidget(self.ui)
    else:
        self.ui = ui()
        self.ui.setupUi(self)

    self.fadeEffect = FadeEffect(self, 100, self.close)

    Shadow(self.ui.container)

    self.ui.cancle.clicked.connect(self.click("cancle"))
    self.ui.confirm.clicked.connect(self.click("confirm"))
    self.ui.btn_close.clicked.connect(self.fadeEffect.close)

    self.setTopic(msg_dict["topic"])
    self.setTitle(msg_dict["title"])
    self.setContent(msg_dict["content"])

    # 设置弹出窗口的位置
    # self.setGeometry(pos[0], pos[1], 200, 100)
    self.setWindowFlags(self.windowFlags() | Qt.WindowStaysOnTopHint)  # 设置窗口置顶

    self.setWindowFlag(Qt.FramelessWindowHint)  # 去除原来的边框
    self.setAttribute(Qt.WA_TranslucentBackground)  # 透明背景


  def click(self, name):
    def func():
      print("点击了：", name)
      if name != "close":
        self.callback.run(name)
      self.fadeEffect.close()
    return func
  

  def show(self):
    self.fadeEffect.show()
    super().show()


  # 输入相对坐标
  def setPos(self,pos_main_win):
    pos=[pos_main_win[0]+580, pos_main_win[1]+70]
    self.setGeometry(pos[0], pos[1], 224, 324)


  # 设置用户名
  def setTopic(self, text):
    self.setWindowTitle(text)


  # 设置用户名
  def setTitle(self, text):
    self.ui.msg_title.setText(text)


  # 设置用户名
  def setContent(self, text):
    self.ui.msg_content.setText(text)


  def setStyle(self,url,color):
    if url and color:
      self.ui.setStyleSheet("""
        background-color: """+color+""";
        border-image: url('"""+url+"""');
        border-radius: 16px;
      """)
    elif url:
      self.ui.setStyleSheet("""
        border-image: url('"""+url+"""');
        border-radius: 16px;
      """)
    elif color:
      self.ui.setStyleSheet("""
        background-color: """+color+""";
        border-radius: 16px;
      """)
    else:
      self.ui.setStyleSheet("""
        background-color: #6a5acd;
        border-radius: 16px;
      """)


  def windowStateChanged(self, oldState, newState):
    super().windowStateChanged(oldState, newState)
    if newState:
      print("Window lost focus")  # 输出窗口失去焦点


  def closeEvent(self, event: QCloseEvent) -> None:
    self.callback.run("close")
    return super().closeEvent(event)



import os,sys
from PySide6.QtWidgets import QApplication
from PySide6.QtGui import QIcon
from app_page_core import Store, Param
from .core import ThreadManager, PageManager, Page, Device, Setting, MainWindow
from .components import StackManager
from .config import *
from .utils import setAppStyle, assetsPath
from .example import Skin
from .apprcc_rc import *

# 绑定顶部右侧按钮
class BindsRightTop(Page):
  def binds(self):
    return {
      "clicked": [
        ("btn_skin", lambda :self.tips('点击了皮肤', 'success')),
        ("btn_login_icon", lambda :self.tips('点击了登录图标', 'success')),
        ("btn_login_text", lambda :self.tips('点击了登录名称', 'success')),
        ("btn_setting", lambda :self.tips('点击了设置', 'success')),
        ("btn_message", lambda :self.tips('点击了消息', 'success')),
      ]
    }

# 需要加载的页面
pages = {
  "chat": Page,
  "setting": Page,
  "skin": Skin,
  "message": Page,
}

# 页面配置项列表
pageOptionList = [
  {
    "name": "测试页面1",
    "id": "chat",
    "filter": "leftBar",
    "icon": "assets/icon/leftbar/chat.png",
    "stack_id": "app_page_chat",
    "right_menu":[
      {"name": "移除", "icon": assetsPath('menu', 'remove.png')},
      {"name": "设置", "icon": assetsPath('menu', 'setting.png')}
    ]
  },
  {
    "name": "测试页面2",
    "id": "setting",
    "filter": "leftBar",
    "icon": "assets/icon/right_top_bar/setting.png",
    "stack_id": "app_page_setting"
  },
  {
    "name": "主题",
    "id": "skin",
    "filter": "right_top_bar",
    "icon": "assets/icon/right_top_bar/skin.png",
    "stack_id": "app_page_skin"
  },
  {
    "name": "信息",
    "id": "message",
    "filter": "right_top_bar",
    "icon": "assets/icon/right_top_bar/message.png",
    "stack_id": "app_page_message"
  }
]

def loadStackPages(target:Page):
  """加载页面

  Args:
      target (Page): 页面对象
  """
  stack_id:str = Setting.getSetting('stack_id')
  filter_id:str = Setting.getSetting('filter_id')
  button_frame_id:str = Setting.getSetting('button_frame_id')
  button_container_id:str = Setting.getSetting('button_container_id')
  pages:dict = Setting.getSetting('pages')
  pageOptionList:list = Setting.getSetting('pageOptionList')
  
  # 挂载到页面管理器
  target.pageManager.mount(target.ui[stack_id], pages=pages, pageOptionList=pageOptionList)
  # 创建导航
  stackManager = StackManager({
    "stack_id": stack_id,
    "filter_id": filter_id,
    "pageOptionList": pageOptionList,
    "button_frame_id": button_frame_id,
    "button_container_id": button_container_id,
  })
  stackManager.initPage()
  bindsRightTop = BindsRightTop()
  bindsRightTop.initPage()
  target.children.add("bindsRightTop", bindsRightTop)
  target.children.add("stackManager", stackManager)

def createApp(SETTING:dict):
  """创建应用
  Args:
      SETTING (dict): 设置字典, 参数如下>>>
          stack_id (str): 栈组件id
          pages (dict): 页面字典
          pageOptionList (list): 页面配置项列表
          filter_id (str): 过滤器id，用来筛选当前显示的页面
          button_frame_id (str): 按钮框架id
          button_container_id (str): 按钮容器id
          button_close_id (str): 关闭按钮id
          button_login_id (str): 登录按钮id
          button_name_id (str): 按钮名称id
          APP_ICON_PATH (str): 应用图标路径
          APP_TITLE (str): 应用标题
          APP_VERSION (str): 应用版本
          IS_DEBUG (bool): 是否调试模式
          PING_HOST (str): 网络连接检查地址
          tips_ui (str|Ui_Form): 提示提示消息ui路径或Ui_Form类
          tipsBox_ui (str|Ui_Form): 提示提示框ui路径或Ui_Form类
          loading_icon (str): 加载图标路径
          small_page_icon (str): 缩小图标路径
          maximize_page_icon (str): 最大窗口图标路径
  """
  # 应用默认配置
  Setting.applySetting({
    'stack_id': 'stackedWidget',
    'pages': pages,
    'pageOptionList': pageOptionList,
    'filter_id': filter_id,
    'button_frame_id': button_frame_id,
    'button_container_id': button_container_id,
    'button_close_id': button_close_id,
    'button_login_id': button_login_id,
    'button_name_id': button_name_id,
    'APP_ICON_PATH': APP_ICON_PATH,
    'APP_TITLE': APP_TITLE,
    'APP_VERSION': APP_VERSION,
    'IS_DEBUG': IS_DEBUG,
    'PING_HOST': PING_HOST,
    'tips_ui': tips_ui,
    'tipsBox_ui': tipsBox_ui,
    'loading_icon': loading_icon,
    'small_page_icon': small_page_icon,
    'maximize_page_icon': maximize_page_icon,
  })
  # 应用用户配置
  Setting.applySetting(SETTING)
  # 创建应用，添加图标
  app = QApplication(sys.argv)
  app.setWindowIcon(QIcon(APP_ICON_PATH))  # 生成exe时改为绝对路径
  # 创建全局参数对象
  param = Param(filePath=None, default=Device.defaultSystemConfig(version=APP_VERSION))
  user_param = Param(os.path.join(param.get("userPath", ""), "user.json"), {})
  system_param = Param(os.path.join(param.get("systemPath", ""), "system.json"), {})
  # 创建程序主窗口
  main_win = MainWindow(system_param)
  threadManager = ThreadManager()
  pageManager = PageManager()
  # 创建全局变量
  Store({
    'app': app,
    'main_win': main_win,
    'ui': main_win.ui,
    'threadManager': threadManager,
    'pageManager': pageManager,
    'param': param,
    'user_param': user_param,
    'system_param': system_param,
  })
  root = Page('root')
  avatar_url = param.pathJoin("userPath", 'images/avatar.png')
  main_win.ui[Setting.getSetting('button_close_id')].clicked.connect(lambda: root.closeApp())
  main_win.ui[Setting.getSetting('button_login_id')].setStyleSheet(f'image: url({avatar_url})')
  main_win.ui[Setting.getSetting('button_name_id')].setText(user_param.get('userInfo/name', "未命名")[:3])
  # 设置样式，必须在创建全局变量之后
  setAppStyle(root)
  # 挂载栈页面
  loadStackPages(root)
  # 根页面初始化
  root.setup()
  # 显示窗口
  main_win.show()
  print(f"----APP_TITLE:{APP_TITLE} APP_VERSION:{APP_VERSION}----")
  # 运行APP
  n = app.exec()
  try:
      sys.exit(n)
  except SystemExit:
      sys.exit(n)



# 记录管理器
class Record:
  def __init__(self, record_num=50) -> None:
    self.current_index = -1
    self.record_num = record_num
    self.record = []

  # 添加记录
  def addRecord(self, id):
    self.current_index = -1
    if len(self.record) >= self.record_num:
      self.record.pop(0)
    self.record.append(id)

  # 根据序号获取记录
  def getRecord(self, index=False):
    if index:
      return self.record[index]
    else:
      return self.record[-1]

  # 左侧记录
  def leftRecord(self):
    max = len(self.record) - 1
    if self.current_index == -1:
      self.current_index = max
    if self.current_index > 0:
      self.current_index = self.current_index - 1
    else:
      self.current_index = 0
    return False if self.current_index == -1 or len(self.record)==0 else self.record[self.current_index]

  # 右侧记录
  def rightRecord(self):
    max = len(self.record) - 1
    if self.current_index == -1:
      self.current_index = max
    if self.current_index < max:
      self.current_index = self.current_index + 1
    else:
      self.current_index = max
    return False if self.current_index == -1 else self.record[self.current_index]



import time
from PySide6.QtCore import QThread, Signal

def default_function(*args):
  # 此处调用函数
  print("6秒后返回数据")
  time.sleep(6)
  return {"data","测试数据"}

class EasyThread(QThread):
  response = Signal(object)
  def __init__(self, function=None):
    super().__init__()
    self.function = function if function else default_function

  staticmethod
  def kill(self, Is_Wait=True):
    if Is_Wait:
      self.wait()
    else:
      self.quit()
    self.terminate()
    self.payload = None
    self.function = None
    self.response = None
    if self.isFinished():  # 如果线程还在运行
      del self

  def setPayload(self, payload):
    self.payload = payload

  def run(self):
    if hasattr(self, "payload"):
      ret = self.function(self.payload)
    else:
      ret = self.function()
    self.response and self.response.emit(ret)

class Waiting_time(EasyThread):
  def __init__(self,second):
    super().__init__()
    self.second = second

  def run(self):
    # print(f'即将在{self.second}s后尝试')
    time.sleep(self.second)
    self.response.emit(True)

class ThreadManager:
  def __init__(self):
    self.threadList = []

  # dict = {"thread", "function", "callback", "payload", "id"}
  def add(self, dict, isRun=False):
    thread = "thread" in dict and dict["thread"]
    func = "function" in dict and dict["function"]
    callback = "callback" in dict and dict["callback"]
    payload = "payload" in dict and dict["payload"]
    
    # 如果没有thread, 创建thread
    if not thread:
      thread = EasyThread(func)
      dict["thread"] = thread
    # 如果有payload，给thread设置payload
    if payload:
      thread.setPayload(payload)
    # 如有有callback，绑定返回信号给callback函数
    if callback:
      # print("绑定callback：", callback)
      thread.response.connect(callback)
    self.threadList.append(dict)
    isRun and thread.start()

  def run(self, id, payload=None):
    item = self.getOne(id)
    if item:
      if payload:
        # 如果有参数使用当前的参数
        item["thread"].setPayload(payload)
      else:
        # 如果没有参数使用初始化的参数
        if "payload" in item:
          item["thread"].setPayload(item["payload"])
      # 运行线程
      item["thread"].start()
    else:
      print("没有可用线程")

  def get(self, id):
    if hasattr(id, '__call__'):
      # 如果id是规则函数，直接传入
      return list(filter(id, self.threadList))
    elif isinstance(id, str):
      # 如果id是字符串，与每项的id进行比较
      return list(filter(lambda x:x.get("id")==id, self.threadList))

  def getOne(self, id):
    threadList = self.get(id)
    if len(threadList)>0:
      item = threadList.pop()
      return item
    else:
      return False

  def remove(self, id=None, param="SYSTEM"):
    if not id:
      threadList = self.threadList
      for each in threadList:
        print('移除线程：',each["id"] if "id" in each else "NULL", "  param:", param)
        thread = each["thread"]
        if hasattr(thread, "destroy"):
          try:
            thread.destroy(param)
          except Exception as e:
            print("============>关闭线程失败 e",e, "    id=", id)
        try:
          thread.response.disconnect()      # 解除信号与回调函数的连接
        except Exception as e:
          print("解除信号与槽的绑定失败 e:", e)
        if hasattr(thread, "ENABLE"):
          thread.kill(False)
        else:
          thread.kill()
        self.threadList = []
    else:
      threadList = self.get(id)
      for each in threadList:
        print('移除线程：', each["id"] if "id" in each else "NULL", "  param:", param)
        thread = each["thread"]
        if hasattr(thread, "destroy"):
          try:
            thread.destroy(param)
          except Exception as e:
            print("============>关闭线程失败 e",e, "    id=", id)
        try:
          thread.response.disconnect()      # 解除信号与回调函数的连接
        except Exception as e:
          print("解除信号与槽的绑定失败 e:", e)
        if hasattr(thread, "ENABLE"):
          thread.kill(False)
        else:
          thread.kill()
        self.threadList.remove(each)


# 事件总线
class EventBus(object):
  def __init__(self, widgets:dict={}, isSendSelf=True):
    self.widgets:dict = widgets
    self.isSendSelf = isSendSelf

  def getWidget(self, id:str):
    return self.widgets.get(id, None)

  def register(self, id:str, signal:str, callback):
    widget = self.getWidget(id)
    # 将对象的__dict__属性储存为一个字典
    widget_dict = widget.__dict__
    connect = widget_dict[signal].connect
    if self.isSendSelf:
      connect(lambda *args: callback(self, *args))
    else:
      connect(callback)

  def clear(self):
    keys = list(self.widgets.keys())
    for key in keys:
      del self.widgets[key]
    
    self.widgets = {}


import os, sys
from PySide6.QtUiTools import QUiLoader
from PySide6.QtCore import QFile, QIODevice
from PySide6.QtWidgets import QWidget
from app_page_core import Store
from ..config import default_theme

def assetsPath(*args):
    """获取资源路径
    Args:
        args (tuple): 目录，文件名

    Returns:
        path (str): 资源绝对路径
    """
    return os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "assets", *args)


def loadUI(filePath, target=None):
  ui_file = QFile(filePath)
  if not ui_file.open(QIODevice.ReadOnly):
    print(f"cannot open {filePath}")
    sys.exit(-1)
  if target:
    return QUiLoader(target).load(ui_file)
  else:
    return QUiLoader().load(ui_file)


# 根据参数设置样式
def setAppStyle(target):
  setting = target.param.child(target.param.pathJoin("userPath", "setting.json"), default_theme)
  current = setting.get("skin/current_skin_index", 0)
  style = setting.get("skinStyle", default_theme['skinStyle'])[current]
  setWidgetStyleById(id='frame_header', style={"background-color": style['header_bg_color']}, cover=True)
  setWidgetStyleById(id='frame_main', style={
    "background-color": style['main_bg_color'],
    "border-image": f"url('{style['app_bg_image'].replace('\\', '/')}') stretch"
  }, cover=True)


def setWidgetStyleById(id:str, style:dict, cover:bool = False):
  store = Store()
  ui = store.get('ui', None)
  if not ui:
    raise Exception("ui not found")

  config = {"styleSheetList":[]}
  try:
    if not cover:
      config["styleSheetList"] = ui[id].styleSheet().split('\n')
  except:
    pass
  ret = f'#{id}'+'{'+ ";".join([key+":"+style[key] for key in style.keys()]) + '}'
  config["styleSheetList"].append(ret)
  styleText = '\n'.join(config["styleSheetList"])
  ui[id].setStyleSheet(styleText)


def setWidgetStyle(widget:QWidget, style:dict|list, id=None, cover:bool = False):
  if isinstance(style, list):
    style = cascading_styles(*style)
  
  config = {"styleSheetList":[]}
  try:
    if not cover:
      config["styleSheetList"] = widget.styleSheet().split('\n')
  except:
    pass

  if id:
    ret = f'#{id}'+'{'+ ";".join([key+":"+style[key] for key in style.keys()]) + '}'
  else:
    ret = ";".join([key+":"+style[key] for key in style.keys()])
  config["styleSheetList"].append(ret)
  style_str = '\n'.join(config["styleSheetList"])
  widget.setStyleSheet(style_str)


def cascading_styles(*args):
  """
  级联样式
  """
  style = {}
  for arg in args:
    if isinstance(arg, dict):
      for key in arg.keys():
        style[key] = arg[key]
    else:
      pass
  return style


def layout_clear(layout):
  while layout.count():
    child = layout.takeAt(0)
    if child.widget() is not None:
      # print("delete widget", child.widget())
      child.widget().deleteLater()
    elif child.layout() is not None:
      # layout_clear(child.layout())
      print("delete layout", child.layout())
      child.layout().deleteLater()


from PySide6.QtCore import QTimer, QPropertyAnimation, QEasingCurve
from PySide6.QtWidgets import QWidget, QScrollArea


def scroll_to_bottom(parent:QWidget|None, scroll_area:QScrollArea|str):
  def scroll():
    # 如果是字符串，寻找滚动区域
    if isinstance(scroll_area, str):
      _scroll_area = parent.findChild(QWidget, scroll_area)
    else:
      _scroll_area = scroll_area
    
    height = _scroll_area.verticalScrollBar().maximum()
    _scroll_area.verticalScrollBar().setValue(height)

  # 创建一个定时器
  QTimer.singleShot(100, scroll)


def smoothScrollToBottom(parent:QWidget|None, scroll_area:QScrollArea|str):
    # 如果是字符串，寻找滚动区域
    if isinstance(scroll_area, str):
      scrollArea = parent.findChild(QWidget, scroll_area)
    else:
      scrollArea = scroll_area  
  
    # 获取滚动条的最大值  
    maxValue = scrollArea.verticalScrollBar().maximum()  
    # 获取当前滚动条的值  
    currentValue = scrollArea.verticalScrollBar().value()
    
    print(maxValue, currentValue)

    # 创建动画
    animation = QPropertyAnimation(scrollArea.verticalScrollBar(), b"value")  
    animation.setDuration(200)  # 设置动画持续时间为500毫秒  
    animation.setStartValue(currentValue)  
    animation.setEndValue(maxValue)  
    animation.setEasingCurve(QEasingCurve.OutQuad)  # 设置缓动曲线  
    animation.start(QPropertyAnimation.DeleteWhenStopped)  # 开始动画，并在结束时删除动画对象
    
    scrollArea.__scroll_to_bottom_animation = animation


# app_page/core/__init__.py
from .EventBus import EventBus
from .EventHook import EventHook
from .MoveEventMechine import MoveEventMechine
from .Page import Page
from .Thread import EasyThread, Waiting_time, ThreadManager
from .MainWindow import MainWindow
from .FileParam import FileParam
from .Record import Record
from .PageManager import PageManager
from ..core import Setting, Device


__all__ = [
  "Device",
  "Setting",
  "EventBus",
  "Record",
  "EventHook",
  "PageManager",
  "MoveEventMechine",
  "Page",
  "EasyThread",
  "ThreadManager",
  "Waiting_time",
  "MainWindow",
  "FileParam"
]


from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget, QMenu
from PySide6.QtGui import QIcon
from ..core.MoveEventMechine import MoveEventMechine

class CallMenu:
  def __init__(self, target:QWidget, itemList:list, type:str='left'or'right'or'both') -> None:
    self.target = target if target else QWidget()
    self.itemList = itemList
    self.initMenu(type)
    self.initAnimation()
    if type != 'left':
      self.target.contextMenuEvent = self.contextMenuEvent
  
  def show(self, event):
    pos = event.globalPos()
    self.target._left_contextMenu.exec_(pos)

  def contextMenuEvent(self, event):
    pos = event.globalPos()
    self.target._right_contextMenu.exec_(pos)
  
  def initAnimation(self):
    pass

  def initMenu(self, type:str):
    menu = QMenu(self.target)
    if type == 'left':
      self.target._left_contextMenu = menu
    elif type == 'right':
      self.target._right_contextMenu = menu
    elif type == 'both':
      self.target._left_contextMenu = menu
      self.target._right_contextMenu = menu
    else:
      raise Exception("type error")

    for each in self.itemList:
      try:
        menu.addAction(QIcon(each["icon"]), each["name"], each["callback"])
      except Exception as e:
        print("initMenu error: ", e)
    self.itemList = []
    
    menu.setWindowFlag(Qt.NoDropShadowWindowHint)  # 去除阴影效果
    menu.setWindowFlag(Qt.FramelessWindowHint)  # 去除原来的边框
    menu.setAttribute(Qt.WA_TranslucentBackground)  # 透明背景

    menu.setStyleSheet(
      """
      QMenu {
        border: 1px solid #f0f0f0; /* 边框宽度为1px，颜色为#CCCCCC */
        border-radius: 10px; /* 边框圆角 */
        background-color: #FAFAFC; /* 背景颜色 */
        font-size: 14px;
        color: #666;
        padding: 10px 0px 10px 0px; /* 菜单项距菜单顶部边界和底部边界分别有5px */

      }

      QMenu::item { /* 菜单子控件item，为菜单项在default的状态 */
        border: 0px solid transparent;
        background-color: transparent;
        min-width: 100px;
        min-height: 30px;
        padding: 0px 10px;
      }

      QMenu::icon { padding: 0px 10px; }

      QMenu::item:selected { /* 为菜单项在selected的状态 */
        background-color: #EDEDEF;
      }

      QMenu::item:disabled{ /* 为菜单项在disabled的状态 */
        color: #CCCCCC;
        background: none;
      }

      QMenu::separator { /* 菜单子控件separator，定义菜单项之间的分隔线 */
        height: 1px;
        background: #CCCCCC;
        margin-left: 2px; /* 距离菜单左边界2px */
        margin-right: 2px; /* 距离菜单右边界2px */
      }

      QMenu::right-arrow { /* 菜单子控件right-arrow，定义子菜单指示器 */
        width: 24px;
        height: 24px;
        image: url(:/Resource/right_arrow);
      }

      QMenu::left-arrow { /* 菜单子控件left-arrow，定义子菜单指示器 */
        width: 24px;
        height: 24px;
        image: url(:/Resource/left_arrow);
      }

      /*
      QMenu::icon:checked {
        border: none;
        background-color: transparent;
        position: absolute;
        width: 24px;
        height: 24px;
      }*/

      QMenu::item::indicator { /* 菜单项子控件indicator，定义菜单项在选中状态下的指示器 */
        width: 24px;
        height: 24px;
      }

      QMenu::item::indicator:unchecked { /* 定义菜单项未选中的状态 */
        image: none;
      }

      QMenu::item::indicator:checked { /* 定义菜单项选中的状态 */
        image: url(:/Resource/checkebox);
      }

      QPushButton#PlayerButton { /* 自定义菜单项中的按钮 */
        border: none;
        background-color: transparent;
      }

      QPushButton#PlayerButton:hover {
        padding-top: 2px;
        padding-left: 2px;
      }

      QPushButton#PlayerButton:pressed {
        padding: 0px;
      }

      """
      )
    


class Click_Menu:
  def __init__(self, target:QWidget, itemList:list, type:str='left'or'right'or'both'):
    self.right_click = CallMenu(target, itemList, type)
    self.mechine = MoveEventMechine(target, self.right_click.show, type)
    target.mousePressEvent = self.mechine.start
    target.mouseMoveEvent = self.mechine.clear
    target.mouseReleaseEvent = self.mechine.stop
  
  def destroy(self):
    self.mechine.target.mousePressEvent = None
    self.mechine.target.mouseMoveEvent = None
    self.mechine.target.mouseReleaseEvent = None
    self.mechine.target = None
    self.mechine.callback = None
    self.mechine.isClick = None
    self.mechine = None
    self.right_click = None



class RightClick_Menu:
  def __init__(self, target:QWidget, itemList:list):
    self.right_click = CallMenu(target, itemList, type='right')

  def destroy(self):
    self.right_click = None



class Click_RightClick_Menu:
  def __init__(self, target:QWidget, itemList:list):
    self.right_click = CallMenu(target, itemList, type='both')
    self.mechine = MoveEventMechine(target, self.right_click.show, type="both")
    target.mousePressEvent = self.mechine.start
    target.mouseMoveEvent = self.mechine.clear
    target.mouseReleaseEvent = self.mechine.stop
  
  def destroy(self):
    self.mechine.target.mousePressEvent = None
    self.mechine.target.mouseMoveEvent = None
    self.mechine.target.mouseReleaseEvent = None
    self.mechine.target = None
    self.mechine.callback = None
    self.mechine.isClick = None
    self.mechine = None
    self.right_click = None


from PySide6.QtCore import Qt
from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QPushButton
from ..core import Page
from ..animation import RightClick_Menu
from ..config import default_theme
from ..utils import loadUI, setAppStyle, select_image, assetsPath


class SkinCard(Page):
  def __init__(self, option) -> None:
    super().__init__(True)
    self.option = option
    self.skin = loadUI(assetsPath('skin_card.ui'))
    layout = QVBoxLayout()
    layout.setAlignment(Qt.AlignTop)
    self.pick = QPushButton("", self.skin)
    self.pick.setFixedSize(260, 120)
    RightClick_Menu(self.pick, [
      {"name": "分享", "callback": self.shareSkin, "icon": assetsPath('menu', 'share.png')},
      {"name": "收藏", "callback": self.loveSkin, "icon": assetsPath('menu', 'love.png')},
      {"name": "更换壁纸", "callback": self.changeImage, "icon": assetsPath('menu', 'pictures.png')},
    ])
    layout.addWidget(self.pick)
    self.setStyle()

  def setup(self):
    super().setup()

    def callback():
      self.callback.run("pick", self.option)
    self.skin.status.clicked.connect(callback)
    self.pick.clicked.connect(callback)

  def setStyle(self):
    option = self.option
    self.pick.setStyleSheet("background-color: transparent;")
    self.skin.setStyleSheet("""#container {
      border-radius: 12px;
      border-image: url('"""+option['app_bg_image']+"""') stretch;
    }""")
    color = "red" if option["id"] == option["current"] else "transparent"
    self.skin.status.setStyleSheet("""
      background-color: """+color+""";
      border:1px solid #c0c0c0;
    """)
    self.skin.name.setText(option["name"])

  def changeStyle(self):
    pass

  def changeImage(self):
    output, _ = select_image(self.skin, assetsPath('skin'))
    
    # 保存背景图片路径
    setting = self.param.child(self.param.pathJoin("userPath","setting.json"), default_theme)
    skinStyle = setting.get("skinStyle")
    skinStyle[self.option["index"]]["app_bg_image"] = output
    setting.save()
    setAppStyle(self)
    self.callback.run("update")

  def loveSkin(self):
    self.tips("已收藏", "success")

  def shareSkin(self):
    self.tips("正在开发")


class Skin(Page):
  def __init__(self):
    super().__init__()
    self.name = "Skin"

  def setup(self):
    super().setup()
    color = ["rgb(255,58,58)", "rgba(255, 0, 0, 0.08)", "rgba(255, 0, 0, 0.05)"]
    self.ui.btn_skin_base.setStyleSheet(f"color: {color[0]}; border: 1px solid {color[1]}; background-color: {color[2]};")
  

  def reset(self):
    # 卡片layout是否被创建，若创建返回里面的卡片数量
    try:
      item_list = list(range(self.skin_layout.count()))
      self.children.remove()
    except:
      item_list = False

    # 若果卡片被创建
    if not item_list == False:
      item_list.reverse()# 倒序删除，避免影响布局顺序
      for i in item_list:
        item = self.skin_layout.itemAt(i)
        if item.widget():
          item.widget().deleteLater()
    else:
      self.skin_layout = QHBoxLayout(self.ui.skin_container)
      self.skin_layout.setAlignment(Qt.AlignTop)


  def show(self, name=None):
    self.update()

  def hide(self, name=None):
    self.children.remove()


  def update(self):
    self.reset()
    setting = self.param.child(self.param.pathJoin("userPath","setting.json"), default_theme)
    skin = setting.get("skin", None)
    self.current_skin_index = skin["current_skin_index"] if skin else 0
    
    def getSkinList():
      current = self.current_skin_index
      skinStyle = setting.get("skinStyle")
      skinStyle['current'] = skinStyle[current].id
      return skinStyle

    def pick(option):
      if self.current_skin_index == option["index"]:
        return
      self.current_skin_index = option["index"]
      setting.set("skin/current_skin_index", option["index"])
      setting.save()
      setAppStyle(self)
      self.reset()
      render()
    
    def render():
      i = 0
      skinList = getSkinList()
      for each in skinList:
        skincard = SkinCard(each)
        skincard.setup()
        skincard.callback.add("pick", pick)
        skincard.callback.add("update", self.update)
        self.children.add(f"skincard_{i}", skincard)
        self.skin_layout.addWidget(skincard.skin)
        i+=1

    render()


import numpy as np
from PySide6.QtWidgets import QMainWindow, QWidget
from PySide6 import QtCore
from PySide6 import QtGui
from app_page import Param
from ..core.EventHook import EventHook

class MoveWin(QMainWindow):
  def __init__(self, target:QMainWindow | QWidget, system_param:Param | None, id:str | None):
    super().__init__(target)
    target.mousePressEvent = self.mousePressEvent
    target.mouseMoveEvent = self.mouseMoveEvent
    target.mouseReleaseEvent = self.mouseReleaseEvent
    self.target = target
    self.target.center = self.center

    self.id = id
    self.m_flag = False
    self.system_param = system_param
    self.mouseMoveEventHook = EventHook(10)

    if self.system_param:
      self.window_position = system_param.get(id, None)
      if self.window_position and len(self.window_position) == 4:
          print('根据已有参数设置窗口位置', self.window_position)
          try:
            self.target.setGeometry(*self.window_position)
          except Exception as e:
            print("设置窗口位置失败！！！ e=",e)
      else:
          print('自动设置窗口位置')
          self.center()
          rect = self.target.geometry()
          self.window_position = [rect.left(),rect.top(),rect.width(),rect.height()]
          print('window_position:', self.window_position)


  # 将窗口移动屏幕中央
  def center(self):
      qr = self.target.frameGeometry()
      cp = QtGui.QGuiApplication.primaryScreen().availableGeometry().center()
      qr.moveCenter(cp)
      self.target.move(qr.topLeft())


  def mousePressEvent(self, event):
    if event.button() == QtCore.Qt.LeftButton:
      self.m_flag = True
      self.m_Position = event.globalPos() - self.target.pos()  # 获取鼠标相对窗口的位置
      event.accept()
      self.target.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))  # 更改鼠标图标


  def mouseMoveEvent(self, mouse_event):
    if QtCore.Qt.LeftButton and self.m_flag:
      self.target.move(mouse_event.globalPos() - self.m_Position)   # 更改窗口位置
      self.mouseMoveEventHook.run(mouse_event.globalPos() - self.m_Position)
      mouse_event.accept()


  def mouseReleaseEvent(self, mouse_event):
    self.m_flag = False
    self.target.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
    self.saveCurrentPosition()


  # 保存位置参数
  def saveCurrentPosition(self):
      if not self.system_param:
        return
      rect = self.target.geometry()
      new_window_position = [rect.left(),rect.top(),rect.width(),rect.height()]
      isMove = not (np.array(self.window_position) == np.array(new_window_position)).all()
      if isMove:
          self.window_position = new_window_position
          self.system_param.set(key=self.id, value=self.window_position)



from PySide6.QtWidgets import QWidget
from PySide6.QtCore import QPropertyAnimation

class FadeEffect:
  def __init__(self, target:QWidget, duration=200, finish=lambda: print("FadeEffect>finish")) -> None:
    if hasattr(target, "_fade_animation"):
      # 如果已经有对象则跳过
      return
    
    target._fade_animation = QPropertyAnimation(target, b'windowOpacity')
    target._fade_animation.setDuration(duration)  # 持续事件1秒

    self.target = target
    self.finish = finish

  def show(self):
    self.target._fade_animation.stop()
    self.target._fade_animation.setStartValue(0)
    self.target._fade_animation.setEndValue(1)
    self.target._fade_animation.start()


  def close(self):
    self.target._fade_animation.stop()
    self.target._fade_animation.finished.connect(self.finish)
    self.target._fade_animation.setStartValue(1)
    self.target._fade_animation.setEndValue(0)
    self.target._fade_animation.start()



import os, hashlib
from app_page_core import Param

def md5_file(path):
# 打开文件，并以二进制模式读取
  with open(path, 'rb') as f:
      # 创建一个md5对象
      md5 = hashlib.md5()
      
      # 逐块读取文件内容，并更新到md5对象中
      for chunk in iter(lambda: f.read(4096), b""):
          md5.update(chunk)
      
      # 返回计算出的md5哈希值
      return md5.hexdigest()


class FileParam: 
  def __init__(self, filePath) -> None:
    self.filePath = os.path.abspath(filePath)
    self.param = Param(os.path.join(filePath, "info.json"))

  def info(self):
    total_size = 0
    for foldername, subfolders, filenames in os.walk(self.filePath):
      for filename in filenames:
        file_path = os.path.join(foldername, filename)
        md5 = md5_file(file_path)
        size = os.path.getsize(file_path)
        total_size += size
        dir_name = self.filePath.split("\\")[-1]
        ret_path = file_path.replace(self.filePath, dir_name).replace("\\","/")
        self.param.set(ret_path, [md5, size])
    
    # size_mb = total_size/1024/1024
    # print(f"fileSize={size_mb:.3f}Mb")
    return self.param.data
  

  def walk(self, callback=None):
    for foldername, subfolders, filenames in os.walk(self.filePath):
      for filename in filenames:
        file_path = os.path.join(foldername, filename)
        md5 = md5_file(file_path)
        size = os.path.getsize(file_path)
        dir_name = self.filePath.split("\\")[-1]
        ret_path = file_path.replace(self.filePath, dir_name).replace("\\","/")
        callback and callback({
          "ret_path": ret_path,
          "md5": md5,
          "size": size,
          "ext": os.path.splitext(filename)[1],
          "filename": filename
        })
    

  def update(self):
    # self._size = 0
    # zip = ZIP()
    # zip.setMode("7z")
    # def download(path, value):
    #   _dir_name, full_file_name = os.path.split(path)
    #   dir_name = _dir_name.replace("dist","zip")
    #   file_name, file_ext = os.path.splitext(full_file_name)
    #   dist_path = os.path.join(dir_name, file_name+"."+zip.mode)
    #   if not os.path.exists(dir_name):
    #     os.makedirs(dir_name)
    #   if zip.zip_file(path, dist_path):
    #     print("压缩成功...")
    #   else:
    #     print("压缩失败...")

    #   # if md5_file(path) == value[0]:
    #   #   # print("path:", path, value[0])
    #   #   print("文件已存在")
    #   # else:
    #   #   print("正在下载 path:", path, value[0])
    #   #   self._size += value[1]

    # self.param.walk(pick=download, type="dict", path="dist/main")
    # size_kb = self._size/1024
    # print(f"下载的文件大小{size_kb:.3f}kb")
    pass


# app_page/utils/__init__.py
from .common import assetsPath, loadUI, setAppStyle, setWidgetStyle, setWidgetStyleById, layout_clear
from .cut_image import cut_image_in, cut_image_out
from .select_image import select_image


__all__ = [
  'loadUI',
  'setAppStyle',
  'setWidgetStyle',
  'setWidgetStyleById',
  'layout_clear',
  'select_image',
  'cut_image_in',
  'cut_image_out',
  'assetsPath'
]


import os
import openpyxl as op
import tempfile
from openpyxl import Workbook
from openpyxl.styles import Font, PatternFill
from openpyxl.styles import Alignment
from openpyxl.drawing.image import Image as XLImage
from PIL import Image

# 提取列部分
def get_column_name(cell_position):
    return ''.join(filter(str.isalpha, cell_position))

def get_resize_image_path(image_path, width, height):
    image = Image.open(image_path)
    image.thumbnail((width, height))
    temp_dir = tempfile.mkdtemp()  # 创建临时目录
    name = os.path.basename(image_path)
    temp_file_path = os.path.join(temp_dir, name)  # 创建临时文件路径
    image.save(temp_file_path)

    return temp_file_path


class Transform:
    def __init__(self, pixels=0):
        self.pixels = pixels
        
    def pixels_to_points(self, pixels, dpi=96):
        # 默认情况下，假设显示器分辨率为 96 ppi (像素/英寸)
        # 如果你的显示器分辨率不同，可以传入 dpi 参数进行调整
        inches = pixels / dpi
        points = inches * 72
        return points

    def points_to_cm(self, points):
        # 在 96 dpi 的分辨率下，一个 12 磅的字体大致等于 0.16875 英寸
        inches = points / 72  # 1 磅 = 1/72 英寸
        cm = inches * 2.54
        return cm

    def pixels_to_cm(self, pixels):
        points = self.pixels_to_points(pixels)
        return self.points_to_cm(points)
    
    def cm_to_pixels(self, cm, dpi=96):
        inches = cm / 2.54
        points = inches * 72
        pixels = points * dpi / 72
        return pixels

    @property
    def cm(self):
        return self.pixels_to_cm(self.pixels)
    @cm.setter
    def cm(self, value):
        self.pixels = self.cm_to_pixels(value)

    @property
    def points(self):
        return self.pixels_to_points(self.pixels)
    @points.setter
    def points(self, value):
        self.pixels = value * 96 / 72

def fill(color):
  return PatternFill(start_color=color, end_color=color, fill_type='solid')


class Excel:
  def __init__(self, filePath=None) -> None:
    self.filePath = filePath
    self.work_book = None
    self.work_sheet = None
    self.cell_data = None
    self.cell_max_len_dict = {}
    
  def load(self, filePath=None, data_only=False):
    if not filePath:
      self.work_book = op.load_workbook(self.filePath, data_only=data_only) if self.filePath and os.path.exists(self.filePath) else Workbook()
    else:
      self.filePath = filePath
      self.work_book = op.load_workbook(filePath, data_only=data_only)
    return self
  
  def sheet(self, name:str='', create=False):
    if name == '':
      self.work_sheet = self.work_book.active
    else:
      if create:
        self.work_sheet = self.work_book[name] if name in self.work_book.sheetnames else self.work_book.create_sheet(name)
      else:
        self.work_sheet = self.work_book[name] if name in self.work_book.sheetnames else None
    return self
    
  def cell(self, row:int | str, column:int | None = None):
    try:
      if row and not column:
        self.cell_data = self.work_sheet[row]
      else:
        self.cell_data = self.work_sheet.cell(row, column)
    except: 
      pass
    return self
  
  def cell_value(self, data=None):
    if data:
      self.cell_data.value = data
      return self
    elif isinstance(data, int) and data == 0:
      self.cell_data.value = data
      return self
    else:
      return self.cell_data.value
    
  def cell_align_center(self):
    self.cell_data.alignment = Alignment(horizontal='center', vertical='center')
    return self

  def cell_auto_column(self, length=None, mode:str='loose' or 'tight'):
    cell_position = self.cell_data.coordinate
    column_name = get_column_name(cell_position)
    _len = length if length else len(str(self.cell_data.value))
    if column_name in self.cell_max_len_dict:
      self.cell_max_len_dict[column_name] = max(self.cell_max_len_dict[column_name], _len)
    else:
      self.cell_max_len_dict[column_name] = _len
    loose_width = (self.cell_max_len_dict[column_name]+2)*1.2      # 设置缓冲
    tight_width = self.cell_max_len_dict[column_name]              # 不设置缓冲
    self.work_sheet.column_dimensions[column_name].width =  loose_width if mode == 'loose' else tight_width
    return self
    
  def cell_font_color(self, color:str):
    if color == "red":
      self.cell_data.font = Font(color="ff0000")
    elif color == "green":
      self.cell_data.font = Font(color="0cb336")
    elif color == "yellow":
      self.cell_data.font = Font(color="#FFFF00")
    elif color == "blue":
      self.cell_data.font = Font(color="0000ff")
    else:
      self.cell_data.font = Font("333333")
    return self
  
  def cell_bg_color(self, color:str):
    if color == "red":
      self.cell_data.fill = fill(color="ff0000")
    elif color == "green":
      self.cell_data.fill = fill(color="0cb336")
    elif color == "yellow":
      self.cell_data.fill = fill(color="#FFFF00")
    elif color == "blue":
      self.cell_data.fill = fill(color="0000ff")
    else:
      self.cell_data.fill = fill("333333")
    return self
    
  def cell_add_image(self, image_path:str, size_w_h:list=[128,128], resize_rate=0):
    width = Transform(size_w_h[0])
    height = Transform(size_w_h[1])
    
    # 在指定单元格插入图片
    if resize_rate <= 0:
        xl_image = XLImage(Image.open(image_path))
    else:
        new_image_path = get_resize_image_path(image_path, width.pixels*resize_rate, height.pixels*resize_rate)
        xl_image = XLImage(Image.open(new_image_path))
    xl_image.width = width.pixels
    xl_image.height = height.pixels
    
    self.work_sheet.add_image(xl_image, self.cell_data.coordinate)

    # 设置单元格大小，使其适应图片
    self.work_sheet.row_dimensions[self.cell_data.row].height = height.points
    self.work_sheet.column_dimensions[self.cell_data.column_letter].width = width.points / 6  # 大约 7.5 个字符宽度适合一张图片

    return self

  def save(self, name:str):
    self.work_book.save(f'{name}.xlsx')



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QVBoxLayout, QWidget
from PySide6.QtGui import QGuiApplication, QCloseEvent, QFont, QFontMetrics
from app_page_core import Callback
from ..animation import FadeEffect
from ..core import Setting
from ..utils import loadUI
from ..config import tips_ui


color_dict = {
    'success': '#28be28',
    'fail': '#e64035',
    'warning': '#eaa640',
    'default': '#4d4d4d'
}

class Tips(QWidget):
    def __init__(self, message, type='default', pos=None):
        super().__init__()
        self.callback = Callback()

        ui = Setting.getSetting("tips_ui", tips_ui)
        # 如果ui是字符串
        if isinstance(ui, str):
            self.ui = loadUI(ui)
            # 创建垂直布局
            layout = QVBoxLayout(self)
            layout.addWidget(self.ui)
        else:
            self.ui = ui()
            self.ui.setupUi(self)

        self.setWindowTitle("提示消息")
        self.ui.tips.setText(message)
        self.ui.tips.setStyleSheet(f'background-color: {color_dict[type]};')
        width, height = self.calculate_text_rect(message, 300, 1)
        self.width = width
        self.height = height

        self.ui.tips.setFixedWidth(width + 20)
        self.ui.tips.setFixedHeight(height + 20)
        self.setFixedWidth(width + 60)
        self.setFixedHeight(height + 60)

        self.setPos(pos)
        # 设置弹出窗口的位置
        # self.setGeometry(pos[0], pos[1], 200, 100)
        self.setWindowFlags(self.windowFlags() | Qt.WindowStaysOnTopHint)  # 设置窗口置顶

        self.setWindowFlag(Qt.FramelessWindowHint)  # 去除原来的边框
        self.setAttribute(Qt.WA_TranslucentBackground)  # 透明背景

        # 添加淡入淡出的动画
        self.fadeEffect = FadeEffect(self, 200, self.close)
        self.fadeEffect.show()     

    def setPos(self, pos):
        if not pos:
            rect = QGuiApplication.primaryScreen().availableGeometry()
            pos = [rect.width()/2, rect.height()/2]
        self.setGeometry(int(pos[0]-(self.width+60)/2), int(pos[1]-(self.height+60)/2), self.width+60, self.height+60)

    def showEvent(self, event):
        # 在窗口显示时关闭定时器
        self.startTimer(2000)  # 设置定时器，2秒后关闭窗口

    def timerEvent(self, event):
        # 定时器触发时关闭窗口
        self.killTimer(event.timerId())
        self.fadeEffect.close()

    def closeEvent(self, event: QCloseEvent) -> None:
        self.callback.run("close")
        return super().closeEvent(event)
    
    # 计算矩形宽高
    def calculate_text_rect(self, text, max_width, line_spacing_factor):
        # 设置字体，确保与实际显示的字体相匹配
        font = QFont()
        font.setPointSize(12)  # 设置字号
        self.setFont(font)

        # 使用QFontMetrics测量文本的边界矩形
        metrics = QFontMetrics(font)

        # 分割文本，计算每一行的宽度和高度
        lines = text.split('\n')
        line_widths = [metrics.boundingRect(line).width() for line in lines]
        max_line_width = max(line_widths)
        total_height = sum(metrics.boundingRect(line).height() for line in lines)

        # 如果宽度小于等于最大宽度，直接返回实际宽度和高度
        if max_line_width <= max_width:
            return max_line_width, total_height

        # 如果宽度大于最大宽度，计算需要的行数，并返回新的宽度和高度
        num_lines = (max_line_width + max_width - 1) // max_width
        new_width = max_width
        new_height = num_lines * total_height * line_spacing_factor

        return new_width, new_height


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main-neweVJXBO.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QHBoxLayout, QLabel,
    QLineEdit, QMainWindow, QMenuBar, QPushButton,
    QScrollArea, QSizePolicy, QSpacerItem, QStackedWidget,
    QVBoxLayout, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1080, 753)
        font = QFont()
        font.setFamilies([u"\u5fae\u8f6f\u96c5\u9ed1"])
        font.setPointSize(10)
        font.setBold(True)
        font.setItalic(False)
        MainWindow.setFont(font)
        MainWindow.setStyleSheet(u"* {\n"
"  padding:0;\n"
"  margin:0;\n"
"  border:none;\n"
"  outline: none;\n"
"}\n"
"\n"
"#btn_change {\n"
"  padding: 4px;\n"
"}\n"
"\n"
"QFrame {\n"
"  margin: 0;\n"
"  border: 0;\n"
"}\n"
"\n"
"/*\u4e0b\u62c9\u9009\u62e9\u6846\u6837\u5f0f*/\n"
"QComboBox{\n"
"  font-size:14px;\n"
"  padding: 3px 5px;\n"
"  border:1px solid rgba(228,228,228,1);\n"
"  border-radius:5px;\n"
"}\n"
"QComboBox:hover {\n"
"  background-color: #E8F5E9;\n"
"}\n"
"\n"
"/*\u4e0b\u62c9\u5217\u8868*/\n"
"QComboBox::drop-down {\n"
"  subcontrol-origin: padding;\n"
"  subcontrol-position: top right;\n"
"  width: 20px;\n"
"  border:none;\n"
"}\n"
"\n"
"/*\u6dfb\u52a0\u7bad\u5934*/\n"
"QComboBox::down-arrow {\n"
"  image: url(./assets/icons/cil-arrow-bottom.png);\n"
"}\n"
"QComboBox QAbstractItemView{\n"
"	background:rgba(255,255,255,1);\n"
"  border:1px solid rgba(228,228,228,1);\n"
"  border-radius:0px 0px 5px 5px;\n"
"  font-size:14px;\n"
"  outline: 0px;\n"
"}\n"
"\n"
"QComboBox QAbstractItemView::item{\n"
"	height:28px;\n"
"	color:#6"
                        "66666;\n"
"	padding-left:9px;\n"
"	background-color:#FFFFFF;\n"
"}\n"
"QComboBox QAbstractItemView::item:hover{ \n"
"  background-color:#409CE1;\n"
"  color:#ffffff;\n"
"}\n"
"QComboBox QAbstractItemView::item:selected{\n"
"  background-color:#409CE1;\n"
"  color:#ffffff;\n"
"}\n"
"\n"
"\n"
"\n"
"\n"
"\n"
"/*QTabWidget*/\n"
"QTabWidget::pane{\n"
"  border:none;\n"
"}\n"
"\n"
"/* \u8bbe\u7f6etabBar\u53f3\u79fb\u8ddd\u79bb */\n"
"QTabWidget::tab-bar {\n"
"  /* left: 5px; */\n"
"}\n"
"/* \u8bbe\u7f6etabBar\u6837\u5f0f */\n"
"QTabBar::tab {\n"
"  background: #fff;\n"
"  border: none;\n"
"  min-width: 60px;\n"
"  height: 24px;\n"
"  font-size: 16px;\n"
"  font-family: '\u5fae\u8f6f\u96c5\u9ed1';\n"
"  border-bottom: 3px solid #ddd;\n"
" }\n"
"\n"
"/*\u9009\u4e2dtabBar\u9009\u4e2d\u65f6\u5019\u80cc\u666f\u8272*/\n"
"QTabBar::tab:selected{\n"
"  background-color: #fff;\n"
"  color: #333;\n"
"  border-bottom: 3px solid skyblue;\n"
"}\n"
"/*\u9009\u4e2dtabBar\u9009\u4e2d\u65f6\u5019\u80cc\u666f\u8272*/\n"
"QTabBar::tab"
                        ":!selected{\n"
"  background-color: #ddd;\n"
"  color: #999;\n"
"}\n"
"\n"
"/*\u56db\u4e2a\u4e0b\u5c5e\u754c\u9762*/\n"
"#tab_baoyang,#tab_qiangxiu,#tab_xiaoxiu,#tab_specific{\n"
"  border: none;\n"
"  outline: none;\n"
"  background: #fff;\n"
"}\n"
"#listView_1,listView_2,listView_3,listView_4{\n"
"  border: none;\n"
"  outline: none;\n"
"  background: #fff;\n"
"}\n"
"\n"
"QTableWidget {\n"
"  border: none;\n"
"  outline: none;\n"
"  font-size:16px;\n"
"  background: #fff;\n"
"}\n"
"\n"
"/* \u8868\u683c\u6837\u5f0f\u8bbe\u7f6e */\n"
"/* QTableWidget \u6807\u9898\u5934\u6574\u4e2a\u533a\u57df */\n"
"QHeaderView{\n"
"  /* \u6574\u4e2a\u6807\u9898\u5934\u533a\u57df\u80cc\u666f\u8272 */\n"
"  background-color:transparent;\n"
"}\n"
"/* \u6807\u9898\u5934 \u6bcf\u4e2a\u5355\u72ec\u7684\u6807\u9898\u533a\u57df */\n"
"QHeaderView::section{\n"
"  font-size:16px; \n"
"  padding: 0 5px;               \n"
"  font-family:\"Microsoft YaHei\"; \n"
"  color:#FFFFFF;\n"
"  background:#43A047;/*\u4e2d\u7eff*/\n"
"  background:"
                        "#0D47A1;/*\u6df1\u84dd*/\n"
"  background:#1B5E20;/*\u6df1\u7eff*/\n"
"  background: #B0C4DE;\n"
"  /*background:#4527A0;\u84dd\u8272*/\n"
"  border:none;\n"
"  /* \u6bcf\u4e2a\u6807\u9898\u7684\u5bf9\u9f50\u65b9\u5f0f\uff08\u8c8c\u4f3c\u4e0d\u80fd\u7528\uff09\u3002\n"
"  \u5efa\u8bae\u4f7f\u7528tableWidget->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft | Qt::AlignVCenter)*/\n"
"  text-align:center;               \n"
"  min-height:30px;               \n"
"  max-height:30px;              \n"
"  margin-left:0px;               \n"
"  padding-left:0px;     \n"
"}\n"
"\n"
"/* \u5782\u76f4\u6eda\u52a8\u6761 handle */\n"
"QScrollBar::handle::vertical{\n"
"  background: #ddd;\n"
"  width:8px;\n"
"  border-radius:3px;  \n"
"}\n"
"QScrollBar::handle::vertical:hover{\n"
"  background: #ccc; \n"
"}\n"
"/* \u5782\u76f4\u6eda\u52a8\u6761 \u533a\u57df */\n"
"QScrollBar::vertical{\n"
"  border-color: rgba(255, 255, 255, 10%);\n"
"  width: 8px;\n"
"  border-radius:3px;\n"
"  /* margin: 5px; */\n"
"}\n"
"/* \u5782\u76f4"
                        "\u6eda\u52a8\u6761  handle\u4e0a\u3001\u4e0b\u533a\u57df\uff08\u672a\u88abhandle\u5360\u7528\u7684\u533a\u57df\uff09 */\n"
"QScrollBar::add-page::vertical, QScrollBar::sub-page::vertical,QScrollBar::add-line::vertical,QScrollBar::sub-line::vertical{\n"
"  border:none;\n"
"  outline:none;\n"
"}\n"
"\n"
"/* \u5782\u76f4\u6eda\u52a8\u6761 handle */\n"
"QScrollBar::handle:horizontal{\n"
"  background: #eee;\n"
"  height:8px;\n"
"  border-radius:3px;  \n"
"}\n"
"/* \u5782\u76f4\u6eda\u52a8\u6761 handle */\n"
"QScrollBar::handle:horizontal::hover{\n"
"  background: #ddd; \n"
"}\n"
"/* \u5782\u76f4\u6eda\u52a8\u6761 \u533a\u57df */\n"
"QScrollBar:horizontal{\n"
"  /* border-color: rgba(255, 255, 255, 10%); */\n"
"  height: 8px;\n"
"  border-radius:3px;\n"
"  /* margin: 5px; */\n"
"}\n"
"/* \u5782\u76f4\u6eda\u52a8\u6761  handle\u4e0a\u3001\u4e0b\u533a\u57df\uff08\u672a\u88abhandle\u5360\u7528\u7684\u533a\u57df\uff09 */\n"
"QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal,QScrollBar::add-line:horizonta"
                        "l,QScrollBar::sub-line:horizontal{\n"
"  border:none;\n"
"  outline:none;\n"
"}\n"
"\n"
"")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.centralwidget.setMinimumSize(QSize(960, 720))
        self.verticalLayout = QVBoxLayout(self.centralwidget)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(10, 10, 10, 10)
        self.frame_header = QFrame(self.centralwidget)
        self.frame_header.setObjectName(u"frame_header")
        self.frame_header.setEnabled(True)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_header.sizePolicy().hasHeightForWidth())
        self.frame_header.setSizePolicy(sizePolicy)
        self.frame_header.setMinimumSize(QSize(0, 60))
        self.frame_header.setMaximumSize(QSize(16777215, 60))
        self.frame_header.setStyleSheet(u"#frame_header{\n"
"	padding: 0;\n"
"	margin: 0;\n"
"}\n"
"\n"
"QPushButton {\n"
"	border: none;\n"
"	color: rgb(255,255,255);\n"
"	border-radius: 6px;\n"
"}\n"
"\n"
"QPushButton:hover {\n"
"	background-color: rgba(0,0,0,0.05);\n"
"}")
        self.frame_header.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_header.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.frame_header)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.frame_logo = QFrame(self.frame_header)
        self.frame_logo.setObjectName(u"frame_logo")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.frame_logo.sizePolicy().hasHeightForWidth())
        self.frame_logo.setSizePolicy(sizePolicy1)
        self.frame_logo.setMinimumSize(QSize(0, 60))
        self.frame_logo.setMaximumSize(QSize(150, 60))
        self.frame_logo.setToolTipDuration(0)
        self.frame_logo.setStyleSheet(u"")
        self.frame_logo.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_logo.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout = QHBoxLayout(self.frame_logo)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.label_logo = QLabel(self.frame_logo)
        self.label_logo.setObjectName(u"label_logo")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.label_logo.sizePolicy().hasHeightForWidth())
        self.label_logo.setSizePolicy(sizePolicy2)
        font1 = QFont()
        font1.setFamilies([u"Microsoft YaHei UI"])
        font1.setPointSize(18)
        font1.setBold(True)
        self.label_logo.setFont(font1)
        self.label_logo.setStyleSheet(u"color: #fff;\n"
"font-weight: bold;")
        self.label_logo.setTextFormat(Qt.TextFormat.AutoText)
        self.label_logo.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse)

        self.horizontalLayout.addWidget(self.label_logo)


        self.horizontalLayout_2.addWidget(self.frame_logo)

        self.frame_search = QFrame(self.frame_header)
        self.frame_search.setObjectName(u"frame_search")
        sizePolicy.setHeightForWidth(self.frame_search.sizePolicy().hasHeightForWidth())
        self.frame_search.setSizePolicy(sizePolicy)
        self.frame_search.setMinimumSize(QSize(300, 60))
        self.frame_search.setMaximumSize(QSize(300, 60))
        self.frame_search.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_search.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout_3 = QHBoxLayout(self.frame_search)
        self.horizontalLayout_3.setSpacing(8)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.btn_leftRecord = QPushButton(self.frame_search)
        self.btn_leftRecord.setObjectName(u"btn_leftRecord")
        self.btn_leftRecord.setMinimumSize(QSize(28, 28))
        self.btn_leftRecord.setMaximumSize(QSize(28, 28))
        self.btn_leftRecord.setStyleSheet(u"border-radius: 14px;\n"
"background-color: rgba(0,0,0,0.1);")
        icon = QIcon()
        icon.addFile(u":/icon/assets/icon/left.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_leftRecord.setIcon(icon)

        self.horizontalLayout_3.addWidget(self.btn_leftRecord)

        self.btn_rightRecord = QPushButton(self.frame_search)
        self.btn_rightRecord.setObjectName(u"btn_rightRecord")
        self.btn_rightRecord.setMinimumSize(QSize(28, 28))
        self.btn_rightRecord.setMaximumSize(QSize(28, 28))
        self.btn_rightRecord.setStyleSheet(u"border-radius: 14px;\n"
"background-color: rgba(0,0,0,0.1);")
        icon1 = QIcon()
        icon1.addFile(u":/icon/assets/icon/right.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_rightRecord.setIcon(icon1)

        self.horizontalLayout_3.addWidget(self.btn_rightRecord)

        self.text_search = QLineEdit(self.frame_search)
        self.text_search.setObjectName(u"text_search")
        self.text_search.setMinimumSize(QSize(0, 32))
        self.text_search.setMaximumSize(QSize(16777215, 32))
        self.text_search.setStyleSheet(u"color: #fff;\n"
"font-size:14px;\n"
"padding: 3px 8px;\n"
"border-radius: 16px;\n"
"background-color: rgba(0,0,0,0.08);")

        self.horizontalLayout_3.addWidget(self.text_search)

        self.btn_voice = QPushButton(self.frame_search)
        self.btn_voice.setObjectName(u"btn_voice")
        self.btn_voice.setMinimumSize(QSize(32, 32))
        self.btn_voice.setMaximumSize(QSize(32, 32))
        self.btn_voice.setStyleSheet(u"border-radius: 16px;\n"
"background-color: rgba(0,0,0,0.1);")
        icon2 = QIcon()
        icon2.addFile(u":/icon/assets/icon/voice3.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_voice.setIcon(icon2)

        self.horizontalLayout_3.addWidget(self.btn_voice)


        self.horizontalLayout_2.addWidget(self.frame_search)

        self.frame_useSelect = QFrame(self.frame_header)
        self.frame_useSelect.setObjectName(u"frame_useSelect")
        self.frame_useSelect.setMinimumSize(QSize(400, 0))
        self.frame_useSelect.setMaximumSize(QSize(400, 16777215))
        self.frame_useSelect.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_useSelect.setFrameShadow(QFrame.Shadow.Raised)
        self.frame_useInfo = QFrame(self.frame_useSelect)
        self.frame_useInfo.setObjectName(u"frame_useInfo")
        self.frame_useInfo.setGeometry(QRect(45, 0, 100, 60))
        sizePolicy1.setHeightForWidth(self.frame_useInfo.sizePolicy().hasHeightForWidth())
        self.frame_useInfo.setSizePolicy(sizePolicy1)
        self.frame_useInfo.setMinimumSize(QSize(100, 60))
        self.frame_useInfo.setMaximumSize(QSize(100, 60))
        self.frame_useInfo.setToolTipDuration(0)
        self.frame_useInfo.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_useInfo.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout_7 = QHBoxLayout(self.frame_useInfo)
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.btn_login_icon = QPushButton(self.frame_useInfo)
        self.btn_login_icon.setObjectName(u"btn_login_icon")
        self.btn_login_icon.setMinimumSize(QSize(32, 32))
        self.btn_login_icon.setMaximumSize(QSize(32, 32))
        self.btn_login_icon.setStyleSheet(u"image: url(:/icon/assets/icon/user.png)")

        self.horizontalLayout_7.addWidget(self.btn_login_icon)

        self.btn_login_text = QPushButton(self.frame_useInfo)
        self.btn_login_text.setObjectName(u"btn_login_text")
        self.btn_login_text.setMinimumSize(QSize(0, 32))
        self.btn_login_text.setMaximumSize(QSize(16777215, 32))
        font2 = QFont()
        font2.setPointSize(11)
        self.btn_login_text.setFont(font2)
        self.btn_login_text.setContextMenuPolicy(Qt.ContextMenuPolicy.NoContextMenu)

        self.horizontalLayout_7.addWidget(self.btn_login_text)

        self.frame_app = QFrame(self.frame_useSelect)
        self.frame_app.setObjectName(u"frame_app")
        self.frame_app.setGeometry(QRect(145, 0, 131, 60))
        self.frame_app.setMinimumSize(QSize(0, 60))
        self.frame_app.setMaximumSize(QSize(200, 60))
        self.frame_app.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_app.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout_8 = QHBoxLayout(self.frame_app)
        self.horizontalLayout_8.setSpacing(0)
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.horizontalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.btn_skin = QPushButton(self.frame_app)
        self.btn_skin.setObjectName(u"btn_skin")
        self.btn_skin.setMinimumSize(QSize(32, 32))
        self.btn_skin.setMaximumSize(QSize(32, 32))
        icon3 = QIcon()
        icon3.addFile(u":/icon/assets/icon/skin.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_skin.setIcon(icon3)
        self.btn_skin.setIconSize(QSize(28, 28))

        self.horizontalLayout_8.addWidget(self.btn_skin)

        self.btn_setting = QPushButton(self.frame_app)
        self.btn_setting.setObjectName(u"btn_setting")
        self.btn_setting.setMinimumSize(QSize(32, 32))
        self.btn_setting.setMaximumSize(QSize(32, 32))
        icon4 = QIcon()
        icon4.addFile(u":/icon/assets/icon/setting.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_setting.setIcon(icon4)
        self.btn_setting.setIconSize(QSize(28, 28))

        self.horizontalLayout_8.addWidget(self.btn_setting)

        self.btn_message = QPushButton(self.frame_app)
        self.btn_message.setObjectName(u"btn_message")
        self.btn_message.setMinimumSize(QSize(32, 32))
        self.btn_message.setMaximumSize(QSize(32, 32))
        icon5 = QIcon()
        icon5.addFile(u":/icon/assets/icon/message.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_message.setIcon(icon5)
        self.btn_message.setIconSize(QSize(28, 28))

        self.horizontalLayout_8.addWidget(self.btn_message)

        self.frame_window = QFrame(self.frame_useSelect)
        self.frame_window.setObjectName(u"frame_window")
        self.frame_window.setGeometry(QRect(275, 0, 116, 60))
        self.frame_window.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
        self.frame_window.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_window.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout_9 = QHBoxLayout(self.frame_window)
        self.horizontalLayout_9.setSpacing(0)
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.horizontalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.btn_mini = QPushButton(self.frame_window)
        self.btn_mini.setObjectName(u"btn_mini")
        self.btn_mini.setMinimumSize(QSize(32, 32))
        self.btn_mini.setMaximumSize(QSize(32, 32))
        font3 = QFont()
        font3.setFamilies([u"Microsoft YaHei UI"])
        self.btn_mini.setFont(font3)
        icon6 = QIcon()
        icon6.addFile(u":/icon/assets/icon/minimizing.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_mini.setIcon(icon6)
        self.btn_mini.setIconSize(QSize(28, 28))

        self.horizontalLayout_9.addWidget(self.btn_mini)

        self.btn_change = QPushButton(self.frame_window)
        self.btn_change.setObjectName(u"btn_change")
        self.btn_change.setMinimumSize(QSize(32, 32))
        self.btn_change.setMaximumSize(QSize(32, 32))
        self.btn_change.setStyleSheet(u"padding: 4px;\n"
"margin: 4px;")
        icon7 = QIcon()
        icon7.addFile(u":/icon/assets/icon/outline-maximize-3.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_change.setIcon(icon7)
        self.btn_change.setIconSize(QSize(28, 28))

        self.horizontalLayout_9.addWidget(self.btn_change)

        self.btn_close = QPushButton(self.frame_window)
        self.btn_close.setObjectName(u"btn_close")
        self.btn_close.setMinimumSize(QSize(32, 32))
        self.btn_close.setMaximumSize(QSize(32, 32))
        icon8 = QIcon()
        icon8.addFile(u":/icon/assets/icon/close.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.btn_close.setIcon(icon8)
        self.btn_close.setIconSize(QSize(28, 28))

        self.horizontalLayout_9.addWidget(self.btn_close)


        self.horizontalLayout_2.addWidget(self.frame_useSelect, 0, Qt.AlignmentFlag.AlignRight)


        self.verticalLayout.addWidget(self.frame_header)

        self.frame_main = QFrame(self.centralwidget)
        self.frame_main.setObjectName(u"frame_main")
        self.frame_main.setStyleSheet(u"#frame_main{\n"
"	bacground-color: #fff;\n"
"}")
        self.frame_main.setLineWidth(0)
        self.horizontalLayout_10 = QHBoxLayout(self.frame_main)
        self.horizontalLayout_10.setSpacing(0)
        self.horizontalLayout_10.setObjectName(u"horizontalLayout_10")
        self.horizontalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.frame_left = QFrame(self.frame_main)
        self.frame_left.setObjectName(u"frame_left")
        self.frame_left.setEnabled(True)
        sizePolicy3 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)
        sizePolicy3.setHorizontalStretch(1)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.frame_left.sizePolicy().hasHeightForWidth())
        self.frame_left.setSizePolicy(sizePolicy3)
        self.frame_left.setMinimumSize(QSize(219, 0))
        self.frame_left.setMaximumSize(QSize(219, 16777215))
        self.frame_left.setStyleSheet(u"QLabel {\n"
"	color: rgb(150,150,150);\n"
"	font-size: 13px;\n"
"}\n"
"QPushButton {\n"
"	margin: 1px 10px;\n"
"	font-size: 16px;\n"
"	text-align: left;\n"
"	padding: 0 10px;\n"
"	color: #444;\n"
"	border-radius: 6px;\n"
"}\n"
"QPushButton:hover {\n"
"	background-color: rgba(0,0,0,0.03);\n"
"}\n"
"QPushButton:actived {\n"
"	background-color: rgba(0,0,0,0.03);\n"
"}")
        self.frame_left.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_left.setFrameShadow(QFrame.Shadow.Raised)
        self.frame_left.setLineWidth(0)
        self.verticalLayout_2 = QVBoxLayout(self.frame_left)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.frame_13 = QFrame(self.frame_left)
        self.frame_13.setObjectName(u"frame_13")
        self.frame_13.setMinimumSize(QSize(0, 245))
        self.frame_13.setMaximumSize(QSize(16777215, 255))
        self.frame_13.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_13.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout_11 = QHBoxLayout(self.frame_13)
        self.horizontalLayout_11.setSpacing(0)
        self.horizontalLayout_11.setObjectName(u"horizontalLayout_11")
        self.horizontalLayout_11.setContentsMargins(0, 10, 0, 0)
        self.scrollArea_3 = QScrollArea(self.frame_13)
        self.scrollArea_3.setObjectName(u"scrollArea_3")
        self.scrollArea_3.setStyleSheet(u"#scrollArea_3{\n"
"	background-color: transparent;\n"
"}")
        self.scrollArea_3.setWidgetResizable(True)
        self.leftbar_container = QWidget()
        self.leftbar_container.setObjectName(u"leftbar_container")
        self.leftbar_container.setGeometry(QRect(0, 0, 219, 245))
        self.scrollArea_3.setWidget(self.leftbar_container)

        self.horizontalLayout_11.addWidget(self.scrollArea_3)


        self.verticalLayout_2.addWidget(self.frame_13)

        self.frame_14 = QFrame(self.frame_left)
        self.frame_14.setObjectName(u"frame_14")
        self.frame_14.setMinimumSize(QSize(0, 30))
        self.frame_14.setMaximumSize(QSize(16777215, 30))
        self.frame_14.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame_14.setFrameShadow(QFrame.Shadow.Raised)
        self.label_13 = QLabel(self.frame_14)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(20, 0, 71, 22))
        sizePolicy2.setHeightForWidth(self.label_13.sizePolicy().hasHeightForWidth())
        self.label_13.setSizePolicy(sizePolicy2)
        font4 = QFont()
        font4.setFamilies([u"Microsoft YaHei"])
        self.label_13.setFont(font4)
        self.label_13.setTextFormat(Qt.TextFormat.AutoText)
        self.label_13.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse)

        self.verticalLayout_2.addWidget(self.frame_14)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)


        self.horizontalLayout_10.addWidget(self.frame_left)

        self.frame_right = QFrame(self.frame_main)
        self.frame_right.setObjectName(u"frame_right")
        sizePolicy4 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)
        sizePolicy4.setHorizontalStretch(3)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.frame_right.sizePolicy().hasHeightForWidth())
        self.frame_right.setSizePolicy(sizePolicy4)
        self.frame_right.setStyleSheet(u"QPushButton {\n"
"	font-size: 14px;\n"
"	background-color: #fff;\n"
"	border-radius: 5px;\n"
"}\n"
"\n"
"QLineEdit {\n"
"	border-radius: 5px;\n"
"	padding: 0 5px;\n"
"}\n"
"\n"
"#frame_right{\n"
"	background-color: transparent;\n"
"}")
        self.horizontalLayout_4 = QHBoxLayout(self.frame_right)
        self.horizontalLayout_4.setSpacing(0)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.stackedWidget = QStackedWidget(self.frame_right)
        self.stackedWidget.setObjectName(u"stackedWidget")
        self.page_1 = QWidget()
        self.page_1.setObjectName(u"page_1")
        self.stackedWidget.addWidget(self.page_1)
        self.page_2 = QWidget()
        self.page_2.setObjectName(u"page_2")
        self.stackedWidget.addWidget(self.page_2)

        self.horizontalLayout_4.addWidget(self.stackedWidget)


        self.horizontalLayout_10.addWidget(self.frame_right)


        self.verticalLayout.addWidget(self.frame_main)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1080, 33))
        MainWindow.setMenuBar(self.menubar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.label_logo.setText(QCoreApplication.translate("MainWindow", u"\u5c0f\u7070\u5999\u8bb0", None))
        self.btn_leftRecord.setText("")
        self.btn_rightRecord.setText("")
        self.text_search.setPlaceholderText(QCoreApplication.translate("MainWindow", u"\u641c\u7d22", None))
        self.btn_voice.setText("")
        self.btn_login_icon.setText("")
        self.btn_login_text.setText(QCoreApplication.translate("MainWindow", u"\u767b\u5f55", None))
        self.btn_skin.setText("")
        self.btn_setting.setText("")
        self.btn_message.setText("")
        self.btn_mini.setText("")
        self.btn_change.setText("")
        self.btn_close.setText("")
        self.label_13.setText(QCoreApplication.translate("MainWindow", u"\u6700\u8fd1\u4f7f\u7528", None))
    # retranslateUi




from PySide6.QtWidgets import QPushButton
from PySide6.QtCore import Qt
from PySide6.QtGui import QMouseEvent

class MoveEventMechine:
  def __init__(self, target, callback, type="both"):
    self.target = target
    self.callback = callback
    self.isClick = None
    self.type = type
  def start(self, event:QMouseEvent):
    self.isClick = True
  def clear(self, event:QMouseEvent):
    self.isClick = False
    QPushButton.mouseMoveEvent(self.target, event)
  def stop(self, event:QMouseEvent):
    if self.isClick:
      if self.type == "left" and event.button() == Qt.LeftButton:
        self.callback(event)
      elif self.type == "right" and event.button() == Qt.RightButton:
        self.callback(event)
      elif self.type == "both":
        self.callback(event)
      elif self.type != "left" and self.type != "right" and self.type != "both":
        raise Exception("button must be left or right or both")
      else:
        # print("nothing to do, pass")
        pass


import json
import paho.mqtt.client as mqtt
from nanoid import generate
from ..core.Thread import EasyThread

class MQTT(EasyThread):
  def __init__(self, option):
    super().__init__()
    self.broker = option["broke"] if "broke" in option else 'broker.emqx.io'
    self.port = option["port"] if "port" in option else 1883
    self.topic = option["topic"] if "topic" in option else "user/test/mqtt"
    self.client_id = 'mqtt_Qt_'+generate(size=6)
    self.client = None
    
  def destroy(self, param):
    print("destroy mqtt param:", param)
    self.client.disconnect()
    self.client.loop_stop()

  # 客户端连接
  def connect_mqtt(self):
    def on_connect(client, userdata, flags, rc):
      if rc == 0:
        print("Connected to MQTT Broker!")
        self.response.emit({"status": 0, "msg": "Connected to MQTT Broker!"})
      else:
        print("Failed to connect, return code %d\n", rc)
        self.response.emit({"status": -1, "msg": "Connected to MQTT Broker!"})
    # Set Connecting Client ID
    client = mqtt.Client(client_id=self.client_id)
    client.on_connect = on_connect
    client.connect(self.broker, self.port)
    self.client = client
  
  # 发布消息
  def publish(self, msg):
    payload = {"client_id":self.client_id, "payload":msg}
    result = self.client.publish(self.topic, json.dumps(payload))
    if result[0] == 0:
      print(f"Send `{msg}` to topic `{self.topic}`")
      self.response.emit({"status": 0, "msg": f"Send `{msg}` to topic `{self.topic}`"})
    else:
      print(f"Failed to send message to topic {self.topic}")
      self.response.emit({"status": -1, "msg": f"Failed to send message to topic {self.topic}"})

  # 订阅消息
  def subscribe(self):
    def on_message(client, userdata, msg):
      # print(f"Received `{msg.payload.decode()}` from `{msg.topic}` topic")
      self.response.emit({"status": 1, "msg": msg.payload.decode(), "topic": msg.topic})
    self.client.subscribe(self.topic)
    self.client.on_message = on_message

  # 启动线程
  def run(self):
    self.connect_mqtt()
    self.subscribe()
    self.client.loop_forever()


import os, sys, hashlib, platform
from PySide6.QtGui import QGuiApplication
from win32com.shell import shell
from ..core import Setting
from ..config import PING_HOST

# 获取屏幕信息
def getScreenInfo():
  result = []
  screens = QGuiApplication.screens()
  for each in screens:
    rect = each.geometry()
    result.append({
      'name': each.name(),
      'width': int(rect.width()*1.25),
      'height': int(rect.height()*1.25)
    })
  return result


# 检查网络连接
def internetConnection():
    import subprocess
    host = Setting.getSetting('PING_HOST', PING_HOST)
    # Determine the ping command based on the OS
    param = '-n' if platform.system().lower() == 'windows' else '-c'
    
    # Build the command
    command = ['ping', param, '1', host]
    ret = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    print("ret code:", ret.returncode)
    return True if ret.returncode == 0 else False


# 检查权限
def check_permissions(file_path):
  try:
    # 检查文件是否存在
    if not os.path.exists(file_path):
      print(f"文件 '{file_path}' 不存在")
      return

    # 检查读取权限
    if os.access(file_path, os.R_OK):
      print(f"有读取权限: {file_path}")
    else:
      print(f"没有读取权限: {file_path}")

    # 检查写入权限
    if os.access(file_path, os.W_OK):
      print(f"有写入权限: {file_path}")
    else:
      print(f"没有写入权限: {file_path}")

  except Exception as e:
    print(f"发生错误: {e}")


# 获取软件的版本
def getSoftwareMD5():
  path = os.path.abspath(sys.argv[0])
  print(path)
  # 打开文件，并以二进制模式读取
  with open(path, 'rb') as f:
      # 创建一个md5对象
      md5 = hashlib.md5()
      
      # 逐块读取文件内容，并更新到md5对象中
      for chunk in iter(lambda: f.read(4096), b""):
          md5.update(chunk)
      
      # 返回计算出的md5哈希值
      return md5.hexdigest()


# 获取我的文档
def getDocPath(pathID=5):
  # 默认返回我的文档路径，出错时返回当前工作路径
  try:
    return shell.SHGetFolderPath(0, pathID, None, 0)
  except:
    return os.getcwd()


# 获取默认系统配置
def defaultSystemConfig(version):
  return {
    "version": version,
    "userPath": os.path.join(getDocPath(), "GreatNoteData", "user"),
    "tempPath": os.path.join(getDocPath(), "GreatNoteData", "temp"),
    "systemPath": os.path.join(getDocPath(), "GreatNoteData", "system"),
  }


# 获取平台信息
def platformInfo():
  # 获取操作系统名称
  os_name = platform.system()
  print("操作系统名称:", os_name)

  # 获取操作系统版本号
  os_version = platform.version()
  print("操作系统版本号:", os_version)

  # 获取操作系统发行版
  os_distribution = platform.uname()
  print("操作系统发行版:", os_distribution)

  # 获取计算机的网络名称
  network_name = platform.node()
  print("计算机的网络名称:", network_name)

  # 获取处理器信息
  processor_info = platform.processor()
  print("处理器信息:", processor_info)

  return {
    "os_name": os_name,
    "os_version": os_version,
    "os_distribution": os_distribution,
    "network_name": network_name,
    "processor_info": processor_info,
    "brief": f"{os_name},{os_version}"
  }




from PySide6.QtWidgets import QStackedWidget


class PageManager:
  def __init__(self):
    self.button_dict = {}
    self.page_dict = {}
    self.data = {}

  def mount(self, stackedWidget:QStackedWidget, pages:dict, pageOptionList:list):
    """
    挂载页面管理器

    Args:
        stackedWidget (QStackedWidget): 栈组件
        pages (dict): 页面字典，{页面id:页面对象}
        pageOptionList (list): 页面参数列表 
    """
    # 添加栈
    self.addStack(stackedWidget)
    # 添加页面
    for key in pages.keys():
      value = pages[key]
      self.addPage(key, value)
    # 添加按钮
    for each in pageOptionList:
      key = each.get("id", None)
      if key:
        self.addButton(key, each)

  # 添加栈
  def addStack(self, stack):
    self.stack = stack

  # 添加页面
  def addPage(self, id:str|dict, Page=None):
    if isinstance(id,str):
      if Page:
        self.page_dict[id] = Page
    elif isinstance(id, dict):
      for each in id.keys():
        self.page_dict[each] = id[each]

  # 添加按钮参数
  def addButton(self, id:str, data:dict):
    if isinstance(id, str) and isinstance(data, dict):
      self.button_dict[id] = data

  # 打开页面
  def open(self, id, *args):
    data = {}
    # 点击的页面立即展示
    if id in self.button_dict:
      data["id"] = id
      # 跳转到页面
      param = self.button_dict.get(id, None)
      index = param.get("stack_index", 0)
      self.stack.setCurrentIndex(index)
      if id in self.page_dict:
        # 创建页面对象
        Page = self.page_dict[id]
        current = Page()                # 实例化页面
        current.initPage()             # 初始化页面
        try:
          current["show"](param, *args)        # 展示页面
        except:
          try:
            current["show"](param)
          except:
            try:
              current["show"]()
            except Exception as e:
              pass
        data["current"] = current
      
    # 刚才打开的页面将其隐藏
    if "current" in self.data and self.data["current"]:
      try:
        self.data["current"]["hide"](*args)
      except:
        try:
          self.data["current"]["hide"]()
        except Exception as e:
          pass
    # 将当前页面赋值
    if "id" in data:
      self.data["id"] = data["id"]
    if "current" in data:
      self.data["current"] = data["current"]

  # 销毁页面
  def destroy(self):
    self.page_dict = {}
    self.button_dict = {}
    self.data = {}


from PySide6 import QtCore
from PySide6.QtWidgets import QWidget, QGraphicsDropShadowEffect

class Shadow:
  def __init__(self, target:QWidget):
    self.effect_shadow = QGraphicsDropShadowEffect(target)
    self.effect_shadow.setOffset(0,0) # 偏移
    self.effect_shadow.setBlurRadius(20) # 阴影半径
    self.effect_shadow.setColor(QtCore.Qt.gray) # 阴影颜色
    target.setGraphicsEffect(self.effect_shadow) # 将设置套用到widget窗口中


import time
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget, QVBoxLayout, QPushButton, QFrame
from ..core import Record
from ..animation import RightClick_Menu
from .Stack import Stack
from ..utils import layout_clear


class StackManager(Stack):
  def __init__(self, options:dict):
    """初始化导航

    Args:
        options (dict): 控制参数
            stack_id (str): 栈的id
            filter_id (str): 过滤器id
            pageOptionList (list): 页面选项列表
            button_frame_id (str): 按钮容器的id
            button_container_id (str): 按钮容器的id
    """
    stack_id = options.get("stack_id", None)
    filter_id = options.get("filter_id", None)
    pageOptionList = options.get("pageOptionList", None)
    button_frame_id = options.get("button_frame_id", None)
    button_container_id = options.get("button_container_id", None)
    if not (stack_id and filter_id and pageOptionList and button_frame_id and button_container_id):
      raise Exception("StackManager初始化失败，options参数错误")

    super().__init__(id= stack_id)
    self.filter_id = filter_id
    self.record = Record()
    self.NOREPEAT = True      # 禁止重复
    self.DEBOUNCE = True      # 防抖开启
    self.DEBOUNCE_TIME = 200  # 200毫秒
    self.timestamp = 0
    self.current_btn = ""
    self.pageOptionList = pageOptionList if pageOptionList else []
    self.button_frame_id = button_frame_id
    self.button_container_id = button_container_id
    self._layout = None

  def setup(self):
    super().setup()
    self.button_frame:QFrame = self.ui[self.button_frame_id]
    self.button_container:QWidget = self.ui[self.button_container_id]
    self.createButton(self.pageOptionList)
    self.activeCurrentButton()


  def binds(self):
    return {
      "clicked": [
        ("btn_leftRecord", self.left_record),
        ("btn_rightRecord", self.right_record),
      ]
    }


  # 激活当前按钮
  def activeCurrentButton(self):
    current_id = self.system_param.get('leftBar_current_btn')
    if current_id:
      self.click(current_id, True)()
      return

    self.click(self.pageOptionList[0]["id"], True)()


  # 创建左侧导航栏按钮
  def createButton(self, pageOptionList:list):
    self.pageOptionList = [each for each in pageOptionList if each.get("filter", "") == self.filter_id]
    self.button_frame.setFixedHeight(360)
    layout = self.button_container.layout()
    if layout:
      self.deleteLeftbarButton(layout)
    else:
      layout = QVBoxLayout(self.button_container)
      layout.setAlignment(Qt.AlignTop)
    self._layout = layout
    # 增加新的元素
    for index, each in enumerate(self.pageOptionList):
      # 如果不存在对应的stack页面，创建一个空白的页面
      stack_id = each["stack_id"]
      stack_index = self.getIndexById(stack_id)
      each['stack_index'] = stack_index
      if stack_index == -1:
        stack = QWidget(self.button_frame)
        if "stack_id" in each:
          stack.setObjectName(stack_id)
        new_index = self.count()
        each['stack_index'] = new_index
        self.insertWidget(new_index, stack)

      button = QPushButton(each["name"], self.button_frame)
      button.setObjectName(each["id"])
      button.clicked.connect(self.click(each["id"]))
      button.setFixedHeight(40)

      # 添加右键菜单
      right_menu = []
      index != 0  and right_menu.append({
        "name": "上移",
        "icon": "./assets/icon/menu/up-arrow.png",
        "callback": self.refreshLeftBtn("up", each['id'])
      })
      index != len(self.pageOptionList)-1 and  right_menu.append({
        "name": "下移",
        "icon": "./assets/icon/menu/down-arrow.png",
        "callback": self.refreshLeftBtn("down", each['id'])
      })
      if "right_menu" in each:
        _right_menu = each["right_menu"]
        for e in _right_menu:
          right_menu.append({
            "name":e["name"],
            "icon":e["icon"],
            "callback":self.right_click(e["name"], each["id"])
          })
      RightClick_Menu(button, right_menu)
      # 添加到布局中
      layout.addWidget(button)


  def refreshLeftBtn(self, name, id):
    def fun():
      index = 0
      for each in self.pageOptionList:
        if each['id'] == id:
          break
        index += 1
      if name == "up":
        self.pageOptionList[index], self.pageOptionList[index-1] = self.pageOptionList[index-1], self.pageOptionList[index]
      elif name == "down":
        self.pageOptionList[index], self.pageOptionList[index+1] = self.pageOptionList[index+1], self.pageOptionList[index]
      
      layout_clear(self._layout)
      self.createButton(self.pageOptionList)
    return fun


  # 创建右侧导航栏按钮
  def deleteLeftbarButton(self, layout):
    count = layout.count()
    if layout is not None:
      for i in range(count):
        item = layout.takeAt(count - 1 - i)
        if not item:
          return
        widget = item.widget()
        if widget is not None:
          layout.removeWidget(widget)
          widget.deleteLater()


  # 设置当前激活按钮的样式
  def setActiveStyle(self, id, old_id:str):
    active_style = '#%s {background-color:rgba(0,0,0,0.04);font-weight:bold;font-size:18px}' % id
    styleSheetList = self.button_frame.styleSheet().split('\n')
    # 删除旧的值
    if old_id and styleSheetList[-1].find(f'#{old_id}') > -1:
      styleSheetList.pop()
    styleSheetList.append(active_style)
    # 将新的样式表设置到程序中
    self.button_frame.setStyleSheet('\n'.join(styleSheetList))


  # 点击事件，根据名称打开对应的页面
  def click(self, id, isActive=None):
    def fun(NO_RECORD=False):
      # 判断重复点击
      if not isActive and self.NOREPEAT and self.current_btn == id:
        self.tips("重复点击")
        return
      # 设置防抖
      current = int(time.perf_counter() * 1000)
      if not isActive and self.DEBOUNCE:
        if (current - self.timestamp) < self.DEBOUNCE_TIME:
          self.tips("过于频繁")
          return

      dataList = list(filter(lambda x:x.get("id")==id, self.pageOptionList))
      if len(dataList) > 0:
        index = self.getIndexById(dataList[0]["stack_id"])
        index = index if index != -1 else 0
      else:
        index = 0
      self.setCurrentPage(index)
      self.setActiveStyle(id, old_id=self.current_btn)

      # 保存记录
      if not NO_RECORD:
        self.record.addRecord(id)

      self.current_btn = id
      self.timestamp = current
      self.system_param.set('leftBar_current_btn', id)
      self.navigateTo(id)

    return fun


  def right_click(self, id, page_name):
    return lambda :self.callback.run("right_click", page_name, id)


  def left_record(self):
    id = self.record.leftRecord()
    if id:
      self.click(id)(True) # 不记录


  def right_record(self):
    id = self.record.rightRecord()
    if id:
      self.click(id)(True) # 不记录


from PySide6.QtGui import QMovie
from PySide6.QtWidgets import QLabel, QVBoxLayout, QHBoxLayout, QGridLayout
from ..utils import setWidgetStyle
from ..utils import layout_clear
from ..config import loading_icon

WHITE_STYLE = {
  "border-radius": "20px",
  "padding": "20px",
  "background-color": "#fff"
}
  
  
class Loading:
  def __init__(self, layout:QVBoxLayout|QHBoxLayout|QGridLayout):
    self.layout = layout
  def create(self):
    # 加载GIF文件
    movie = QMovie(loading_icon)
    # movie.setCacheMode(QMovie.CacheAll)
    movie.setSpeed(120)                    # 可调整播放速度，默认为100%
    movie.finished.connect(movie.start)    # 电影结束后重新开始播放
    movie.isValid() and movie.start()      # 开始播放GIF
    # 添加一个label
    loading = QLabel()
    loading.setScaledContents(True)
    loading.setMovie(movie)
    loading.setFixedHeight(300)
    loading.setFixedWidth(300)
    setWidgetStyle(loading, WHITE_STYLE)
    self.layout.addWidget(loading)
    self.movie = movie
    self.loading = loading
  
  def remove(self):
    try:
      print("remove loading")
      self.loading.setMovie(None)
      layout_clear(self.layout)
      self.movie.stop()
      self.movie.finished.disconnect()
      self.movie.deleteLater()
      self.movie = None
      self.layout = None
    except Exception as e:
      print("remove loading error",e)


from PySide6.QtWidgets import QWidget
from PySide6.QtCore import QPropertyAnimation, QPoint

class ShakeEffect:
  def __init__(self, target:QWidget) -> None:
    if hasattr(target, "_shake_animation"):
      # 如果已经有对象则跳过
      return
    
    animation = QPropertyAnimation(target, b'pos', target)
    target._shake_animation = animation
    animation.finished.connect(lambda: delattr(target, '_shake_animation'))

    pos = target.pos()
    x, y = pos.x(), pos.y()

    animation.setDuration(100)
    animation.setLoopCount(2)
    animation.setKeyValueAt(0, QPoint(x, y))
    animation.setKeyValueAt(0.09, QPoint(x + 2, y - 2))
    animation.setKeyValueAt(0.18, QPoint(x + 4, y - 4))
    animation.setKeyValueAt(0.27, QPoint(x + 2, y - 6))
    animation.setKeyValueAt(0.36, QPoint(x + 0, y - 8))
    animation.setKeyValueAt(0.45, QPoint(x - 2, y - 10))
    animation.setKeyValueAt(0.54, QPoint(x - 4, y - 8))
    animation.setKeyValueAt(0.63, QPoint(x - 6, y - 6))
    animation.setKeyValueAt(0.72, QPoint(x - 8, y - 4))
    animation.setKeyValueAt(0.81, QPoint(x - 6, y - 2))
    animation.setKeyValueAt(0.90, QPoint(x - 4, y - 0))
    animation.setKeyValueAt(0.99, QPoint(x - 2, y + 2))
    animation.setEndValue(QPoint(x, y))

    animation.start()


import time

# 毫秒级时间戳
def timestamp():
    return int(time.time() * 1000)

# 事件钩子
class EventHook:
    def __init__(self, delay_ms=10):
        self.pool = {}
        self.delay_ms = delay_ms  # 防抖触发，默认10ms
    
    def add(self, id:str, func:callable):
        self.pool[id] = {
            "func": func,
            "last_run_time": 0  # 初始化上次运行时间为0
        }

    def get(self, id):
        return self.pool.get(id)

    def remove(self, id=None):
        if id is None:
            self.pool.clear()
        else:
            self.pool.pop(id, None)
        return True

    def run(self, *args, **kwargs):
        now = timestamp()
        for id, item in self.pool.items():
            last_run_time = item["last_run_time"]
            if now - last_run_time > self.delay_ms:
                try:
                    item["func"](*args, **kwargs)
                    self.pool[id]["last_run_time"] = now
                except Exception as e:
                    print(f"运行函数失败，ID: {id}，错误: {e}")


import setuptools
from pathlib import Path

package_name = "app-page"
version = '0.0.19'
# 读取 README.md 作为长描述
long_description = open("README.md", encoding="utf-8").read()
# 读取 requirements.txt 文件
requirements_path = Path(__file__).parent / "requirements.txt"
with open(requirements_path, "r") as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name=package_name,
    version=version,
    author="xiaohuicat",  # 作者名称
    author_email="1258702350@qq.com", # 作者邮箱
    description="PySide6 page application framework", # 库描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/xiaohuicat/python-app-page", # 库的官方地址
    license="MIT",
    packages=setuptools.find_packages(),
    package_data={
        'app_page': ['*', 'assets/*', 'assets/*/*'],
    },
    install_requires=requirements,
    zip_safe=False,
)


from ..core import Page

class Stack(Page):
  def __init__(self, id:str='stackedWidget'):
    super().__init__()
    self.id = id

  def setCurrentPage(self, index):
    return self.ui[self.id].setCurrentIndex(index)

  def count(self):
    return self.ui[self.id].count()

  def getPageByIndex(self, index):
    return self.ui[self.id].widget(index)

  def getIndexById(self, id):
    count = self.count()
    for i in range(count):
      if self.getPageByIndex(i).objectName() == id:
        return i
    return -1

  def getPageById(self, id):
    return self.getPageByIndex(self.getIndexById(id))

  def addWidget(self, widget):
    return self.ui[self.id].addWidget(widget)

  def insertWidget(self, index, widget):
    return self.ui[self.id].insertWidget(index, widget)



from PySide6.QtWidgets import QMainWindow, QGraphicsDropShadowEffect
from PySide6.QtGui import QGuiApplication
from PySide6 import QtCore, QtGui
from app_page_core import Param, Callback
from ..animation import MoveWin
from ..config import small_page_icon, maximize_page_icon, APP_TITLE
from ..core import Setting
from ..core.ui_main import Ui_MainWindow


# 程序主窗口的类
class MainWindow(QMainWindow):
    def __init__(self, system_param:Param):
        super().__init__()
        self.system_param = system_param
        self.callback = Callback()
        UI = Setting.getSetting("Ui_MainWindow", Ui_MainWindow)
        # 可以通过[]访问属性的UI 类
        class MYUI(UI):
            def __init__(self,*args,**kwargs):
                    super().__init__(*args,**kwargs)
            def __getitem__(self,__name):
                return super().__getattribute__(__name)

        self.ui = MYUI()  # 使用ui文件导入定义界面类
        self.ui.setupUi(self)  # 初始化界面
        self.normal_window_rect = [570, 79, 1080, 746]
        self.current_window_rect = [*self.normal_window_rect]

        self.setWindowTitle(Setting.getSetting("APP_TITLE", APP_TITLE))
        self.setWindowFlag(QtCore.Qt.FramelessWindowHint)  # 去除原来的边框
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)  # 透明背景
        self.ui.btn_change.clicked.connect(self.restore_or_maximize_window)
        self.ui.btn_mini.clicked.connect(self.showMinimized)
        self.ui.label_logo.setText(Setting.getSetting("APP_TITLE", APP_TITLE))
        self.isMaximized = False

        self.effect_shadow = QGraphicsDropShadowEffect(self)
        self.effect_shadow.setOffset(0,0) # 偏移
        self.effect_shadow.setBlurRadius(16) # 阴影半径
        self.effect_shadow.setColor(QtCore.Qt.gray) # 阴影颜色
        self.ui.centralwidget.setGraphicsEffect(self.effect_shadow) # 将设置套用到widget窗口中

        self.move_win = MoveWin(self, system_param, "main_window_position")
        self.isMaximized = self.move_win.window_position[2] != 1080
        if self.isMaximized:
            self.ui.btn_change.setIcon(QtGui.QIcon(Setting.getSetting("small_page_icon", small_page_icon)))
        else:
            self.ui.btn_change.setIcon(QtGui.QIcon(Setting.getSetting("maximize_page_icon", maximize_page_icon)))
        self.current_window_rect = [*self.move_win.window_position]

    def restore_or_maximize_window(self):  # 放大缩小
        if self.isMaximized:
            self.isMaximized = False
            if self.current_window_rect[2] != 1080:
                self.current_window_rect = [*self.normal_window_rect]
            self.setGeometry(*self.current_window_rect)
            self.ui.btn_change.setIcon(QtGui.QIcon(Setting.getSetting("maximize_page_icon", maximize_page_icon)))
            self.callback.run("restore_window", self.current_window_rect)
        else:
            self.isMaximized = True
            raw = self.geometry()
            self.current_window_rect = [raw.x(),raw.y(),raw.width(),raw.height()]
            rect = QGuiApplication.primaryScreen().availableGeometry()
            self.setGeometry(-10,-10,rect.width()+20,rect.height()+45)
            self.ui.btn_change.setIcon(QtGui.QIcon(Setting.getSetting("small_page_icon", small_page_icon)))
            self.callback.run("restore_window", [0,0,rect.width(),rect.height()])
        self.move_win.saveCurrentPosition()


# app_page/animation/__init__.py
from .FadeEffect import FadeEffect
from .Loading import Loading
from .MoveWin import MoveWin
from .RightClick import RightClick_Menu, Click_RightClick_Menu
from .Shadow import Shadow
from .ShakeEffect import ShakeEffect
from ..animation import ScrollMethod

__all__ = [
  "FadeEffect",
  "Loading",
  "MoveWin",
  "RightClick_Menu",
  "Click_RightClick_Menu",
  "Shadow",
  "ShakeEffect",
  "ScrollMethod"
]


from app_page_core import Store, Param, Callback, LocalStore
from .core import Page, EventBus, EventHook, EasyThread, ThreadManager, PageManager, Waiting_time, Setting, Device, FileParam
from .app import createApp

__all__ = [
  "createApp",
  "Store",
  "Page",
  "PageManager",
  "Param",
  "Children",
  "Callback",
  "LocalStore",
  "EventHook",
  "EventBus",
  "EasyThread",
  "ThreadManager",
  "Waiting_time",
  "Setting",
  "Device",
  "FileParam"
]


import sys
from PySide6.QtWidgets import QApplication
from app_page_core import Page as CorePage
from app_page_core import Param
from ..core.Tips import Tips
from ..core.TipsBox import TipsBox
from ..core.PageManager import PageManager
from ..core.Thread import ThreadManager
from ..core.MainWindow import MainWindow

class Page(CorePage):
  def __init__(self, name=None):
    super().__init__(name)
    self.app:QApplication
    self.main_win:MainWindow
    self.ui:MainWindow.ui
    self.pageManager:PageManager
    self.threadManager:ThreadManager
    self.system_param:Param
    self.user_param:Param

  # 导航到页面
  def navigateTo(self, id, param=None):
    if param:
      self.pageManager.open(id, param)
    else:
      self.pageManager.open(id)

  # 提示信息
  def tips(self, msg, type='default', pos=None, close=None):
    p = self.main_win.move_win.window_position
    pos=[p[0]+p[2]/2, p[1]+p[3]/2]

    if hasattr(self.app, "tips_widget"):
      self.main_win.move_win.mouseMoveEventHook.remove(id="tips")
      self.app.tips_widget.deleteLater()

    self.app.tips_widget = Tips(msg, type, pos)
    self.app.tips_widget.show()
    self.app.tips_widget.callback.add("close", close)

    def win_move(pos):
      self.app.tips_widget.setPos([pos.x()+p[2]/2, pos.y()+p[3]/2])

    self.main_win.move_win.mouseMoveEventHook.add(id="tips", func=win_move)

  # 提示窗
  def tipsBox(self, option, confirm=None, cancle=None, close=None):
    if hasattr(self, "tips_box") and self.tips_box:
      print("请关闭后再打开")
      return
    self.tips_box = TipsBox(self.system_param, option)

    def _close():
      self.tips_box.callback.remove()
      self.tips_box.deleteLater()
      self.tips_box = None
      close and close()

    confirm and self.tips_box.callback.add("confirm", confirm)
    cancle and self.tips_box.callback.add("cancle", cancle)
    self.tips_box.callback.add("close", _close)
    self.tips_box.show()

  # 关闭页面
  def close(self):
    # 销毁挂载的回调函数
    self.callback and self.callback.destroy()
    # 移除子组件
    self.children.remove()

  # 页面初始化
  def initPage(self):
    # 页面初始化
    if hasattr(self, 'setup'):
      self.setup()
    
    # 绑定函数
    if hasattr(self, "binds"):
      bind_dict = self.binds()
      for signal in bind_dict.keys():
        for [id, callback] in bind_dict[signal]:
          widget = self.ui[id]
          # 将对象的__dict__属性储存为一个字典
          widget_dict = widget.__dict__
          widget_dict[signal].connect(callback)

  # 关闭app
  def closeApp(self):
    self.system_param.save()
    self.user_param.save()
    self.threadManager.remove()
    self.children.remove()
    n = self.app.exec()
    try:
      sys.exit(n)
    except SystemExit:
      print('程序退出了，顺手帮你把垃圾带走')
      sys.exit(n)
      
  # 查看组件信息
  @property
  def info(self):
    info_string = f"\n当前页面有{len(self.children.components.keys())}子页面。\n"+self.children.info()
    return info_string